uniform sampler2D sSourceTex;
uniform sampler2D sDepthTex;

// x = UV space offset of one texel width (used for horizontal pass), y = UV space offset of one texel height (used for vertical pass), z = Depth falloff, w = unused
uniform vec4 uBilateralBlurDirectionAndDepthFalloff;

in vec2 vTexCoord;

#include "UndefineInOut.inc"
void AddWeightedSample(vec2 sampleUV, float centralDepth, float gaussianWeight, inout vec4 resultOut, inout float totalWeightOut) // Note the 'in' and the 'out'.  Just using 'out' will default the arguments to 0
{
    float depth = float(texture2DLodCompat(sDepthTex, sampleUV, 0.0).r);
    float weighting = abs(depth - centralDepth) * uBilateralBlurDirectionAndDepthFalloff.z;
    weighting = exp(-weighting * weighting) * gaussianWeight;

    resultOut += texture2D(sSourceTex, sampleUV) * weighting;
    totalWeightOut += weighting;
}
#include "DefineInOut.inc"

void main()
{
    const vec4 weight = vec4(1.0, 0.8, 0.4, 0.1); // Tried gaussian weights but manually tweaked seems better.  The kernel also works when the weights are totally flat, but this is somewhere in the middle.

    vec2 uv = vTexCoord;
    float centralDepth = float(texture2DLodCompat(sDepthTex, uv, 0.0).r);

    vec4 result = texture2D(sSourceTex, uv) * weight.x;
    float totalWeight = weight.x;

    AddWeightedSample(uv + (uBilateralBlurDirectionAndDepthFalloff.xy * -3.0), centralDepth, weight.w, result, totalWeight);
    AddWeightedSample(uv + (uBilateralBlurDirectionAndDepthFalloff.xy * -2.0), centralDepth, weight.z, result, totalWeight);
    AddWeightedSample(uv + (uBilateralBlurDirectionAndDepthFalloff.xy * -1.0), centralDepth, weight.y, result, totalWeight);
    AddWeightedSample(uv + (uBilateralBlurDirectionAndDepthFalloff.xy *  1.0), centralDepth, weight.y, result, totalWeight);
    AddWeightedSample(uv + (uBilateralBlurDirectionAndDepthFalloff.xy *  2.0), centralDepth, weight.z, result, totalWeight);
    AddWeightedSample(uv + (uBilateralBlurDirectionAndDepthFalloff.xy *  3.0), centralDepth, weight.w, result, totalWeight);
    gl_FragColor = result / totalWeight;
}
