#ifdef BRIGHT_PASS_TONEMAP_ENABLED
   uniform float uExposureKey;         /** Value from 0-1 that adjusts the brightness of the final image, otherwise known as middle grey */
   uniform float uMinAutoExposure;     /** Minimum allowed auto exposure */
   uniform float uMinBlackLum;         /** the smallest luminance that will be mapped to pure black */
   uniform float uMaxWhiteLum;         /** the smallest luminance that will be mapped to pure white */
#endif
uniform float uMaxAutoExposure;     /** Maximum allowed auto exposure */
uniform float uBrightnessThreshold; /** Any value below this threshold will be black */

in vec2 vTexCoord;

uniform sampler2D sSceneHDRTex;
uniform sampler2D sAvgLumTex;

void main()
{
    vec3 colourHDR = texture2DLodCompat(sSceneHDRTex, vTexCoord, 0.0).rgb;

    // Determine what the pixel's value will be after tone-mapping occurs
    float colourLum = CalculateLuminance(colourHDR);
#ifdef BRIGHT_PASS_TONEMAP_ENABLED
    float avgLum = UnpackLuminanceFloat(texture2DLodCompat(sAvgLumTex, vec2(0.5, 0.5), 0.0));
    gl_FragColor = (colourLum > uBrightnessThreshold) ? vec4(ToneMap(colourHDR, avgLum, uMinBlackLum, uMaxWhiteLum, uExposureKey, uMinAutoExposure, uMaxAutoExposure), 1.0) : vec4(0.0, 0.0, 0.0, 1.0);
#else
   // We still need to clamp to max exposure as some NVidia cards have MSAA issues that cause values much greater than 1 to come through and we don't want those values bloomed
   gl_FragColor = (colourLum > uBrightnessThreshold) ? vec4(clamp(colourHDR, 0.0, uMaxAutoExposure), 1.0) : vec4(0.0, 0.0, 0.0, 1.0);
#endif
}
