#ifdef COLOUR_CORRECTION_ENABLED

    #ifdef VOLUME_TEXTURES
        uniform sampler3D sColourRemapTex1;
        #ifdef COLOUR_REMAPS_2
        uniform sampler3D sColourRemapTex2;
        #endif
        #ifdef COLOUR_REMAPS_3
        uniform sampler3D sColourRemapTex3;
        #endif
    #else
        uniform sampler2D sColourRemapTex1;
        #ifdef COLOUR_REMAPS_2
        uniform sampler2D sColourRemapTex2;
        #endif
        #ifdef COLOUR_REMAPS_3
        uniform sampler2D sColourRemapTex3;
        #endif
    #endif
    uniform vec4 uColourRemapWeightings;

    #ifdef VOLUME_TEXTURES
        vec3 lutSample(sampler3D lut, vec3 inCoord)
        {
            //Sample straight from 16x16x16 3d texture
            //Put the coord in the 0.5-15.5 range so texels are sampled correctly
            return texture3D(lut, (inCoord * 0.0625 * 15.0) + (0.5 * 0.0625)).rgb;
        }
    #else
        vec3 lutSample(sampler2D lut, vec3 inCoord)
        {
            //Requires a volume texture 16x16x16 unwrapped in a 2d texture 256x16
            const vec2 offset = vec2(0.5 / 256.0, 0.5 / 16.0);
            const float scale = 15.0 / 16.0;
            const float scaleDiv16 = scale / 16.0;
            float intB = floor(inCoord.b * 14.9999) * 0.0625;
            float fracB = (inCoord.b * 15.0) - (intB * 16.0);
            float u = intB + (inCoord.r * scaleDiv16);
            float v = inCoord.g * scale;
            vec3 rg0 = texture2D(lut, offset + vec2(u,v)).rgb;
            vec3 rg1 = texture2D(lut, offset + vec2(u + 0.0625, v)).rgb;
            return mix(rg0, rg1, fracB);
        }
    #endif

    vec3 ColourCorrect(vec3 inColour)
    {
        vec3 colour = inColour * uColourRemapWeightings.x;
        colour += lutSample(sColourRemapTex1, inColour) * uColourRemapWeightings.y;
        #ifdef COLOUR_REMAPS_2
        colour += lutSample(sColourRemapTex2, inColour) * uColourRemapWeightings.z;
        #endif
        #ifdef COLOUR_REMAPS_3
        colour += lutSample(sColourRemapTex3, inColour) * uColourRemapWeightings.w;
        #endif
        return colour;
    }

 #endif

