#include "Shared/DepthUtils.inc"


uniform mat4 uSceneInvViewMatrix;
uniform mat4 uSceneInvProjSunlightViewMatrix;
uniform vec3 uInvViewSpaceSunDirection;
uniform SAMPLER_2D_AUTO_MULTISAMPLE uNormalTexture;
uniform SAMPLER_2D_AUTO_MULTISAMPLE uDepthTexture;

in vec2 vTextureUV;

void main()
{
    // Sunlight shadow attenuation
    float amountInSunlight = 0.0;

#if defined(SUNLIGHT_SHADOWS)
    vec3 viewNormal = texture2DMultisample(uNormalTexture, vTextureUV, uViewportLookupScale.zw).xyz;
    // Skip sky pixels
    if (dot(viewNormal, vec3(1.0)) == 0.0)
    {
        discard;
        return;
    }
    viewNormal = viewNormal * 2.0 - 1.0;

    int iCascade = -1;

    // We only need to perform shadow attenuation test for pixels that are front facing to the sunlight
    // @todo This optimisation currently causes too many artefacts
    //float sunlightNDotL = dot(viewNormal, uInvViewSpaceSunDirection);
    //if (sunlightNDotL > 0.0)
    {
        float nonLinearClipDepth = texture2DMultisample(uDepthTexture, vTextureUV, uViewportLookupScale.zw).r;
        float linearViewSpaceDepth = GetViewSpaceDepth(nonLinearClipDepth, uZBufferParams.xyz);

        // Here we are using the GetWorldSpacePos() to convert the clip space pos at this fragments position into light view space with one matrix
        vec4 worldPos = vec4(0.0, 0.0, 0.0, 0.0);
        vec4 sunlightViewPos = GetWorldSpacePos(vTextureUV, nonLinearClipDepth, uSceneInvProjSunlightViewMatrix);
        vec3 camLook = vec3(uViewMatrix[0][2], uViewMatrix[1][2], uViewMatrix[2][2]);
        amountInSunlight = DirLightShadowAtten(iCascade, worldPos, sunlightViewPos, linearViewSpaceDepth, uSunlightShadowMap, uSunlightShadowTranslucencyMap, 1.0);
    }
#else
    amountInSunlight = 1.0;
#endif

#if defined(SUNLIGHT_SHADOWS) && defined(DEBUG_SUNLIGHT_SHADOW_CASCADE)
    gl_FragColor = vec4(ShadowMapCascadeColour(iCascade, int(uMappingParams.w)).rgb, 1.0);
#else
    gl_FragColor.r = amountInSunlight;

    // Debug normal
    //gl_FragColor = vec4(viewNormal.x, viewNormal.y, viewNormal.z, 1.0);
    //gl_FragColor = vec4(worldNormal * 0.5 + 0.5, 1.0);
#endif
}
