#include "Shared/MultisampleTexture.inc"

// Check effect so this shader can be used both inside and outside RT7 shader system
#if defined(VIEWPORTLOOKUPSCALE)
    //[
    UNIFORM_BUFFER_BEGIN(ViewportLookupScale)
        uniform highp vec4 uViewportLookupScale;
        uniform highp vec4 uViewportOffsetScale;
        uniform highp vec4 uFullScreenLookupScale;
    UNIFORM_BUFFER_END
    //]
#else
   uniform highp vec4 uViewportLookupScale;
   uniform highp vec4 uViewportOffsetScale;
   uniform highp vec4 uFullScreenLookupScale;
#endif

// Default down sample comparator, override by defining DEPTH_DOWNSAMPLE_OP externally
#if !defined(DEPTH_DOWNSAMPLE_OP)
    #define DEPTH_DOWNSAMPLE_OP min
#endif

uniform SAMPLER_2D_AUTO_MULTISAMPLE sDepthTex;

in vec2 vTexCoord;

void main()
{
    float result = 0.0;
    vec4 depths = vec4(0.0, 0.0, 0.0, 0.0);
#if defined(MSAA) && defined(MSAA_AVAILABLE) // TextureGather doesnt work for MSAA samplers, so we need a fallback path :(
    ivec2 intTexCoord = ivec2(vTexCoord * uViewportLookupScale.zw);
    depths.x = texelFetch(sDepthTex, intTexCoord + ivec2(-1, -1), 0).r;
    depths.y = texelFetch(sDepthTex, intTexCoord + ivec2(-1, 1), 0).r;
    depths.z = texelFetch(sDepthTex, intTexCoord + ivec2(1, -1), 0).r;
    depths.w = texelFetch(sDepthTex, intTexCoord, 0).r;
#else
    depths = textureGatherCompat(sDepthTex, vTexCoord, uViewportLookupScale.xy);
#endif
    result = DEPTH_DOWNSAMPLE_OP(depths.x, DEPTH_DOWNSAMPLE_OP(depths.y, DEPTH_DOWNSAMPLE_OP(depths.z, depths.w)));

    gl_FragDepth = result;
}
