uniform sampler2D sSourceTex;
uniform sampler2D sBlurNearTex;
uniform sampler2D sBlurFarTex;
uniform sampler2D sFocusTex;
uniform sampler2D sSourceDepthTex;
uniform vec4 uBokehParams;

in vec2 vTexCoord;

#include "DepthOfFieldShared.inc"

void main()
{
    vec4 focus = texture2DLodCompat(sFocusTex, vTexCoord, 0.0);
    vec3 sourceTexel = texture2DLodCompat(sSourceTex, vTexCoord, 0.0).rgb;

    float near = focus.x;
    vec3 blurNear = sourceTexel;
    if (near > 0.0)
    {
        vec3 blurNearTexel = texture2DLodCompat(sBlurNearTex, vTexCoord, 0.0).rgb;
        // We check again for NaNs as it is possible for the blur stage to output a NaN
        // In this scenario, the least visually broken result is to output an unblurred pixel
        blurNear = isNaN(blurNearTexel.r) ? blurNear : mix(blurNear, blurNearTexel, near);
    }

    if (focus.y > 0.0)
    {
        vec3 blurFarTexel = texture2DLodCompat(sBlurFarTex, vTexCoord, 0.0).rgb;

        // Recompute far focus at full res to avoid aliasing
        float depth = texture2DLodCompat(sSourceDepthTex, vTexCoord, 0.0).r;
        float focalPlane = GetViewSpaceDepth(depth, uZBufferParams.xyz) - uDofFocalPoint;
        float far = smoothstep(uDofParams.z, uDofParams.z + uDofParams.w, focalPlane);

        // Select near/far blur source and blend with full res image
        vec3 blurFar = isNaN(blurFarTexel.r) ? sourceTexel : mix(sourceTexel, blurFarTexel, far);
        vec3 blurTexel = mix(blurNear, blurFar, step(near, far));
        gl_FragColor = vec4(blurTexel, 1.0);
    }
    else
    {
        gl_FragColor = vec4(blurNear, 1.0);
    }
}
