
void main()
{
#if defined(VERTEX_COLOUR)
    vec4 vertexColour = vColour;
#endif

#if defined(DIFFUSE_MAP)
    // TextureUV2D = Interpolated atlas UV incl tiling (i.e. can go > 1 etc.)
    // AtlasMin = Fixed minimum corner atlas UV of sprite we are interested in
    // AtlasExtents = Fixed size in atlas UV space of sprite we are interested in

    const highp float FLT16_EPSILON = 6.10e-5;

    //Extents can be negative, indicating wrapping type, so abs() them before we start to use them
    highp vec2 absExtents = abs(vTextureUVAtlasMin_Extents.zw);

    highp vec2 uvClamp = min(max(vTextureUV2D, vTextureUVAtlasMin_Extents.xy), vTextureUVAtlasMin_Extents.xy + absExtents);
    //Use FLT16_EPSILON to ensure we don't try to mod() by 0.0 (for when minU==maxU or minV==maxV)
    highp vec2 uvWrap = mod(vTextureUV2D - vTextureUVAtlasMin_Extents.xy, max(absExtents,vec2(FLT16_EPSILON))) + vTextureUVAtlasMin_Extents.xy;

    //Use the negative extent values to pick between the wrapped or clamped UVs
    highp vec2 mixStep = step(vec2(FLT16_EPSILON), vTextureUVAtlasMin_Extents.zw);
    //highp vec2 mixStep = step(vec2(0.0), vTextureUVAtlasMin_Extents.zw);
    highp vec2 uv = mix(uvClamp, uvWrap, mixStep);

    vec4 texel;
    #if defined(GUI_MIPMAP)
        texel = texture2D(uDiffuseMap, uv);
    #else
        texel = texture2DLodCompat(uDiffuseMap, uv, 0.0);
    #endif

    #if defined(GUI_MASK)
        vec4 texelMask;
        highp vec2 uvMask = (uv - vTextureUVAtlasMin_Extents.xy) / absExtents;
        #if defined(GUI_MIPMAP)
            texelMask = texture2D(uMaskMap, uvMask);
        #else
            texelMask = texture2DLodCompat(uMaskMap, uvMask, 0.0);
        #endif

        #if defined(GUI_MASK_TRANSITION) && defined(VERTEX_COLOUR)
            //Vertex colour alpha is used to control transition mask strength
            float spread = 0.05;
            float aStart = mix(-spread, 1.0, vertexColour.a);
            float aEnd = mix(0.0, 1.0 + spread, vertexColour.a);

            float a = (texelMask.b - aStart) / (aEnd - aStart);
            a = min(1.0, max(0.0, a));
            texelMask.a *= (1.0 - a);

            //Clear the alpha controling the mask
            vertexColour.a = 1.0;
        #endif

        #if defined(PREMULTIPLY_ALPHA)
            texel.rgb *= texelMask.a;
        #endif

        texel.a *= texelMask.a;
    #endif

    #if defined(VERTEX_COLOUR)
        #if defined(PREMULTIPLY_ALPHA)
            texel.rgb *= vertexColour.a;
        #endif
        // We are checking for the magic no texture value (roughly negative max half float)
        gl_FragColor = mix(vec4(1.0), texel, step(-60000.0, vTextureUV2D.x)) * vertexColour;
    #else
        gl_FragColor = mix(vec4(1.0), texel, step(-60000.0, vTextureUV2D.x));
    #endif

#elif defined(VERTEX_COLOUR)
    gl_FragColor = vertexColour;
#else
    // This path should never be taken
    gl_FragColor = vec4(0,0,0,0);
#endif
}
