#ifdef BLUR_VERTICAL
#define DIRECTION_SWIZZLE yx
#else
#define DIRECTION_SWIZZLE xy
#endif

#define KERNEL_RADIUS 3

uniform SAMPLER_2D_AUTO_MULTISAMPLE uDepthTexture;
uniform sampler2D uSSAOTexture;

uniform highp vec4 uGeometryAwareBlurParams; // near*far, near-far, far, sharpness

in vec2 vTextureUV;

#include "Shared/DepthUtils.inc"


void BlurPass(float ssaoInput, float blurDepth, float centerDepth, int offset, inout float result, inout float weightSum)
{
    const float blurSigma = float(KERNEL_RADIUS) * 0.5;
    const float blurFalloff = 1.0 / (2.0 * blurSigma * blurSigma);

    float linearBlurDepth = GetViewSpaceDepth(blurDepth, uGeometryAwareBlurParams.xyz);
    float deltaDepth = (linearBlurDepth - centerDepth) * uGeometryAwareBlurParams.w;
    float offsetFloat = float(offset);
    float weight = exp2(-offsetFloat * offsetFloat * blurFalloff - deltaDepth * deltaDepth);
    weightSum += weight;
    result += ssaoInput * weight;
}

// This has to be done in this crazy verbose way to keep the AMD compiler happy about constant texture offsets
#if defined(MSAA) && defined(MSAA_AVAILABLE)
    #define BLUR_PASS(offset) BlurPass( \
        textureLodOffset(uSSAOTexture, uv, 0.0, ivec2(offset, 0).DIRECTION_SWIZZLE).r, \
        texelFetch(uDepthTexture, intUV + ivec2(offset, 0).DIRECTION_SWIZZLE, 0).r, \
        linearDepth, offset, result, weightSum);
#else
    #define BLUR_PASS(offset) BlurPass( \
        textureLodOffset(uSSAOTexture, uv, 0.0, ivec2(offset, 0).DIRECTION_SWIZZLE).r, \
        textureLodOffset(uDepthTexture, uv, 0.0, ivec2(offset, 0).DIRECTION_SWIZZLE).r, \
        linearDepth, offset, result, weightSum);
#endif

void main()
{
    // Can't be minified because BLUR_PASS macro needs consistent variable names
    //[
    vec2 uv = vTextureUV * uViewportOffsetScale.zw + uViewportOffsetScale.xy;
    gl_FragColor = texture2DLodCompat(uSSAOTexture, uv, 0.0);
    float result = gl_FragColor.r;
    float depth = texture2DMultisample(uDepthTexture, uv, uFullScreenLookupScale.zw).r;
    // Skip sky pixels
    if (depth == 1.0)
    {
        gl_FragColor = vec4(1.0);
        return;
    }
    float linearDepth = GetViewSpaceDepth(depth, uGeometryAwareBlurParams.xyz);
    float weightSum = 1.0;
    ivec2 intUV = ivec2(uv * uFullScreenLookupScale.zw);

    BLUR_PASS(-3);
    BLUR_PASS(-2);
    BLUR_PASS(-1);
    BLUR_PASS(1);
    BLUR_PASS(2);
    BLUR_PASS(3);

    result /= weightSum;
    gl_FragColor.r = result;
    //]
}
