#include "UndefineInOut.inc"

#if defined(USE_TEXTURE_ARRAY)
    #define KAWASE_SAMPLER_TYPE sampler2DArray
#else
    #define KAWASE_SAMPLER_TYPE sampler2D
#endif

// Performs 4 samples for Kawase filter on red channel of given texture and returns samples individually
vec4 KawaseFilter1(KAWASE_SAMPLER_TYPE tex, vec2 uv, vec2 pixelSize, float iteration, float sourceTexSlice)
{
    vec2 halfPixelSize = pixelSize * 0.5;
    vec2 uvOffset = pixelSize.xy * vec2(iteration, iteration) + halfPixelSize;

    vec2 texCoordSample = vec2(0.0, 0.0);
    /* Array support currently disabled
#if defined(USE_TEXTURE_ARRAY)
    vec3 texCoordSample = vec3(0.0, 0.0, sourceTexSlice);
#else
    vec2 texCoordSample = vec2(0.0, 0.0);
#endif
    */
    vec4 result;

    texCoordSample.x = uv.x - uvOffset.x;
    texCoordSample.y = uv.y + uvOffset.y;
    result.x = texture2DLodCompat(tex, texCoordSample, 0.0).r;

    texCoordSample.x = uv.x + uvOffset.x;
    texCoordSample.y = uv.y + uvOffset.y;
    result.y = texture2DLodCompat(tex, texCoordSample, 0.0).r;

    texCoordSample.x = uv.x + uvOffset.x;
    texCoordSample.y = uv.y - uvOffset.y;
    result.z = texture2DLodCompat(tex, texCoordSample, 0.0).r;

    texCoordSample.x = uv.x - uvOffset.x;
    texCoordSample.y = uv.y - uvOffset.y;
    result.w = texture2DLodCompat(tex, texCoordSample, 0.0).r;

    return result;
}

// As previous but operating on and returning 2 channels (unrotated for efficency)
void KawaseFilter2(KAWASE_SAMPLER_TYPE tex, vec2 uv, vec2 pixelSize, float iteration, out vec4 result12, out vec4 result34, float sourceTexSlice)
{
    vec2 halfPixelSize = pixelSize * 0.5;
    vec2 uvOffset = pixelSize.xy * vec2(iteration, iteration) + halfPixelSize;

    //vec2 texCoordSample;
#if defined(USE_TEXTURE_ARRAY)
    vec3 texCoordSample = vec3(0.0, 0.0, sourceTexSlice);
#else
    vec2 texCoordSample = vec2(0.0, 0.0);
#endif

    texCoordSample.x = uv.x - uvOffset.x;
    texCoordSample.y = uv.y + uvOffset.y;
    result12.xy = texture2DLodCompat(tex, texCoordSample, 0.0).rg;

    texCoordSample.x = uv.x + uvOffset.x;
    texCoordSample.y = uv.y + uvOffset.y;
    result12.zw = texture2DLodCompat(tex, texCoordSample, 0.0).rg;

    texCoordSample.x = uv.x + uvOffset.x;
    texCoordSample.y = uv.y - uvOffset.y;
    result34.xy = texture2DLodCompat(tex, texCoordSample, 0.0).rg;

    texCoordSample.x = uv.x - uvOffset.x;
    texCoordSample.y = uv.y - uvOffset.y;
    result34.zw = texture2DLodCompat(tex, texCoordSample, 0.0).rg;
}

// As previous but operating on and returning 3 channels (unrotated for efficency)
void KawaseFilter3(KAWASE_SAMPLER_TYPE tex, vec2 uv, vec2 pixelSize, float iteration, out vec3 result1, out vec3 result2, out vec3 result3, out vec3 result4, float sourceTexSlice)
{
    vec2 halfPixelSize = pixelSize * 0.5;
    vec2 uvOffset = pixelSize.xy * vec2(iteration, iteration) + halfPixelSize;

    //vec2 texCoordSample;
#if defined(USE_TEXTURE_ARRAY)
    vec3 texCoordSample = vec3(0.0, 0.0, sourceTexSlice);
#else
    vec2 texCoordSample = vec2(0.0, 0.0);
#endif

    texCoordSample.x = uv.x - uvOffset.x;
    texCoordSample.y = uv.y + uvOffset.y;
    result1 = texture2DLodCompat(tex, texCoordSample, 0.0).rgb;

    texCoordSample.x = uv.x + uvOffset.x;
    texCoordSample.y = uv.y + uvOffset.y;
    result2 = texture2DLodCompat(tex, texCoordSample, 0.0).rgb;

    texCoordSample.x = uv.x + uvOffset.x;
    texCoordSample.y = uv.y - uvOffset.y;
    result3 = texture2DLodCompat(tex, texCoordSample, 0.0).rgb;

    texCoordSample.x = uv.x - uvOffset.x;
    texCoordSample.y = uv.y - uvOffset.y;
    result4 = texture2DLodCompat(tex, texCoordSample, 0.0).rgb;
}

#include "DefineInOut.inc"
