#ifndef MATH_UTILS_INC
#define MATH_UTILS_INC


const float PI = 3.14159265358979323846264;
const float INV_PI = 0.31830988618379067153776786954291;
const float TWOPI = PI * 2.0;
const float INV_TWOPI = 1.0 / TWOPI;
const float PI_OVER_4 = PI / 4.0;
const float PI_OVER_2 = PI / 2.0;
const float SQRT_2_PI = 0.797884560802865;
const float INV_EIGHT = 1.0 / 8.0;


/**
    @return Roughness value required by Beckmann NDF, also referred to as 'm' in academic literature
 */
float SpecPowToBeckmannRoughness(float specPow)
{
    return sqrt(2.0 / (specPow + 2.0));
}

/**
    @param Perceptual roughness is roughness in the range 0-1 (i.e. what an artist would paint down)
    @return Academic roughness, commonly referred to as 'm' in literature, which is perceptualRoughness squared.
 */
float PerceptualRoughnessToRoughness(float perceptualRoughness)
{
    return perceptualRoughness * perceptualRoughness;
}

/**
    @param Academic roughness, commonly referred to as 'm' in literature, which is perceptualRoughness squared.
    @return Perceptual roughness is roughness in the range 0-1 (i.e. what an artist would paint down)
 */
float RoughnessToPerceptualRoughness(float roughness)
{
    return sqrt(roughness);
}


#endif
