#include "Shared/ModelUtils.inc"

//Main function
void main()
{
    //Set based on the width/height of the billboard
    vec2 offset = aBillboardSize;

    vec4 worldPosition;
    vec4 cameraSpacePosition;
    //Work out vertex position
    #if defined(ANIMATION_VERTEX)
        // Note uModelMatrix contains a full MVP transform
        mat4 modelMatrix = uModelMatrix;
        mat4 boneMatrix = GetBoneFromIndex(aBone_Label_BillboardLabel_Joint.x);
        mat4 jointMatrix = GetBoneFromIndex(aBone_Label_BillboardLabel_Joint.w);

        worldPosition = uModelMatrix * jointMatrix * boneMatrix * vec4(aVertexPositionDepthOffset.xyz, 1.0);
        cameraSpacePosition = uViewMatrix * worldPosition;
    #else
        worldPosition = uModelMatrix * vec4(aVertexPositionDepthOffset.xyz, 1.0);
        cameraSpacePosition = uViewMatrix * worldPosition;
    #endif
    //Apply billboard specific transforms
    #if defined(ANIMATION_BILLBOARD)
        //Translation
        cameraSpacePosition.xy += uBillboardLabelTransforms[int(aBone_Label_BillboardLabel_Joint.z)].xy;
        //Do scale and rotation separately
        offset.xy *= uBillboardLabelTransforms[int(aBone_Label_BillboardLabel_Joint.z)].zw;
        float rotation = uBillboardLabelTransforms[int(aBone_Label_BillboardLabel_Joint.z)+1].x;

        //Calculate the rotation of the position offset
        float sinTheta = sin(rotation);
        float cosTheta = cos(rotation);
        mat2 rotationMatrix = mat2(cosTheta, sinTheta, -sinTheta, cosTheta);
        offset.xy = rotationMatrix * offset.xy;
    #endif
    //Adjust camera space position using zOffset - cameraSpacePosition is the position of the CENTER of the billboard at this stage.  Look at gl_model.java render_billboards (line 3159) for reference.  Basically this line emulates the translation factor Java places in the matrix.
    cameraSpacePosition.xyz -= normalize(cameraSpacePosition.xyz) * aVertexPositionDepthOffset.w;
    //Add offset to the position (offseting will make cameraSpacePosition the position of the corner vert of the billboard)
    cameraSpacePosition.xy += offset.xy;
    gl_Position = uProjectionMatrix * cameraSpacePosition;
    #if defined(PUSH_TO_FARPLANE)
        gl_Position.z = gl_Position.w * 0.9999;
    #endif
    vVertexAlbedo = aVertexColour;
    #if defined(ANIMATION_BILLBOARD)
        // Animate Transparency
        vVertexAlbedo.a += uLabelDeltas[int(aBone_Label_BillboardLabel_Joint.y)].a;
        vVertexAlbedo.a = clamp(vVertexAlbedo.a, 0.0, 1.0);

        // Animated colour in RGB Space is straight-forward
        #if defined(ANIMATION_COLOUR_RGB)
            vVertexAlbedo.xyz += uLabelDeltas[int(aSamplerWrapAndBoneAndLabel.z)].xyz;
        #endif

        // Animated colour in HSL Space is a bit more convoluted, swap into HSL space, edit, then swap back
        #if defined(ANIMATION_COLOUR_HSL)
            // Animate Colour
            vVertexAlbedo = convertRGBtoHSL(vVertexAlbedo);
            // Adjust HSL of the model, wrapping round for Hue, clamping for Saturation and Lighting
            vVertexAlbedo.x = fract(vVertexAlbedo.x + uLabelDeltas[int(aBone_Label_BillboardLabel_Joint.y)].x);
            vVertexAlbedo.yz = clamp(vVertexAlbedo.yz + uLabelDeltas[int(aBone_Label_BillboardLabel_Joint.y)].yz, 0.0, 1.0);
            vVertexAlbedo = convertHSLtoRGB(vVertexAlbedo);
        #endif
    #endif
    vVertexAlbedo.a += uFade;
    //Change the u and v elements of texture meta to represent the UV coordinates instead of the size
    vMaterialSettingsSlotXY = aMaterialSettingsSlotXY_UV.xy;
    //Get the UVs for this vertex
    vTextureUV = aMaterialSettingsSlotXY_UV.zw;
    #if defined(LIGHT_SCATTERING) || defined(FOG_DISTANCE)
        vec3 viewDirection = worldPosition.xyz - uCameraPosition;
        float cameraDistance = length(viewDirection);

        #if defined(LIGHT_SCATTERING) && defined(SUNLIGHT_DIRECT_LIGHTING)
            ComputeInOutScattering(normalize(viewDirection), cameraDistance, uInvSunDirection.xyz, vOutScattering, vInScattering);
        #else
            vOutScattering = vec3(1.0);
            vInScattering = vec3(0.0);
        #endif

        #if defined(FOG_DISTANCE)
            float fogFactor = FogBasedOnDistance(cameraDistance);
            vInScattering = mix(vInScattering, uFogColour.rgb, fogFactor);
            vOutScattering *= 1.0 - fogFactor;
        #endif
    #endif
}
