//Main function
void main()
{
    vec4 fragColour;

    TextureSettings textureSettings;
    getTextureSettings(vMaterialSettingsSlotXY, textureSettings);

    vec4 texelAlbedo = vec4(1.0);
    vec4 texelEmissiveNormal = vec4(0.0, 0.5, 1.0, 0.5); // 0 emissive (R), no-op normal (GBA)
    vec4 texelMetalnessRoughness = vec4(0.0); //0 metalness (R) and roughness (G)

    getTexel(vTextureUV, textureSettings.textureMeta1, textureSettings.textureMeta2, textureSettings.textureMeta3, uAtlasMeta, textureSettings.wrapping, uTextureAtlas, texelAlbedo, texelEmissiveNormal, texelMetalnessRoughness);

    #if defined(ETC_CHANNEL_SWIZZLE)
        // Swizzle ETC channels back into the expected order
        texelEmissiveNormal = texelEmissiveNormal.agbr; //NX,NY,NZ,HD -> HD,NY,NZ,NX
        texelMetalnessRoughness = texelMetalnessRoughness.rabg; //M,X,X,R -> M,R,X,X
    #endif

    fragColour = texelAlbedo * vVertexAlbedo;
    //TODO remove?
    fragColour *= fragColour; // A pretty necessary hack to compensate for the brightening of unlit objects caused by the 'linear to gamma' stage of tone mapping

    //Throw away any fragments which are below the set threshold for alpha testing
    if (fragColour.a <= uAlphaTestThreshold)
    {
        discard;
        return;
    }

    //Add emissive
    // The colour can be either the albedo or a specified colour in the material settings
    vec3 emissiveColour = mix(fragColour.rgb, textureSettings.emissiveColourAndEmissiveSource.rgb, textureSettings.emissiveColourAndEmissiveSource.a);
    // Scale the colour and add it to itself - this allows for 0 value emissiveScale which equates to being unlit
    emissiveColour += emissiveColour * textureSettings.emissiveScale;
    // Lerp between the lit colour and the emissive colour based on the mask
    fragColour.rgb = mix(fragColour.rgb, emissiveColour, texelEmissiveNormal.r);

    //Light scattering (now includes fog!)
    #if defined(LIGHT_SCATTERING) || defined(FOG_DISTANCE)
        fragColour.xyz = ApplyInOutScattering(fragColour.xyz, vOutScattering, vInScattering);
    #endif

    #if defined(HEAT_HAZE)
        fragColour.rgb = vec3(1.0, 1.0, 1.0);
    #endif

    gl_FragColor = fragColour;
}
