#include "ConversionUtils.inc"
#include "Shared/ModelGeometryUtils.inc"
#include "Shared/ModelGeometryVertexUtils.inc"

void main()
{
    float label = 1.0;
    AssignPositionNormalVaryings(label);

    //Colour
#if defined(TEXTURE_ATLAS)
    #if defined(TEXTURE_ALBEDO_GLOBAL)
        vVertexAlbedo = aVertexColour;
    #else
        // Check forced texture flag if textures are off globally
        // Re-use the alpha from the non-whitened vertex colour here (saves a float in the vertex data)
        vVertexAlbedo = vec4(mix(aVertexColourUnwhitenedRGB_TilePositionLevel.xyz, aVertexColour.xyz, step(0.5, fract(aVertexNormal_BatchFlags.w * 8.0))), aVertexColour.w);
    #endif
#else
    // Re-use the alpha from the non-whitened vertex colour here (saves a float in the vertex data)
    vVertexAlbedo = vec4(aVertexColourUnwhitenedRGB_TilePositionLevel.xyz, aVertexColour.a);
#endif
    vVertexAlbedo.a += uFade;

#if defined(TINT)
    #if !defined(TINT_SHIFT_HSL) && !defined(PUSH_TO_FARPLANE) //In the sky model, the tint behaviour will not do the constant addition we need (and we need it in the fragment shader too)
        // Tinting if set (towards weighting w)
        vVertexAlbedo.rgb = vVertexAlbedo.rgb + uTint.w * (uTint.rgb - vVertexAlbedo.rgb);
    #endif
#endif
#if defined(ANIMATION_VERTEX)
    // Animate transparency
    vVertexAlbedo.a += uLabelDeltas[int(label)].a;
    vVertexAlbedo.a = clamp(vVertexAlbedo.a, 0.0, 1.0);

    #if defined(ANIMATION_COLOUR_RGB) || defined(ANIMATION_COLOUR_HSL)
        // Animated colour in RGB Space is straight-forward
        #if defined(ANIMATION_COLOUR_RGB)
            vVertexAlbedo.xyz += uLabelDeltas[int(label)].xyz;
        #endif
        // Animated colour in HSL Space is a bit more convoluted, swap into HSL space, edit, then swap back
        #if defined(ANIMATION_COLOUR_HSL)
            // Animate Colour
            vVertexAlbedo = convertRGBtoHSL(vVertexAlbedo);
            // Adjust HSL of the model, wrapping round for Hue, clamping for Saturation and Lighting
            vVertexAlbedo.x = fract(vVertexAlbedo.x + uLabelDeltas[int(label)].x);
            vVertexAlbedo.yz = clamp(vVertexAlbedo.yz + uLabelDeltas[int(label)].yz, 0.0, 1.0);
            vVertexAlbedo = convertHSLtoRGB(vVertexAlbedo);
        #endif
        // Correct for gamma
        #if defined(BAKED_SRGB_TO_LINEAR) && !defined(GAMMA_CORRECT_INPUTS)
            vVertexAlbedo.rgb = SRGBToLinear(vVertexAlbedo.rgb);
        #endif
    #endif
#endif

#if !defined(BAKED_SRGB_TO_LINEAR) && defined(GAMMA_CORRECT_INPUTS)
    vVertexAlbedo.rgb = SRGBToLinear(vVertexAlbedo.rgb);
#endif

#if defined(GOURAUD_SHADING)
    float sunlightNDotL = dot(vNormal, uInvSunDirection); // Yes we're not clamping our N.L because that's what Java does :)  - read JavaModel.java lightInner() lines 727-729
    vVertexAlbedo.rgb = vVertexAlbedo.rgb * (uAmbientColour + (uSunColour * sunlightNDotL));
#endif

#if defined(POINT_LIGHTING) && defined(POINT_LIGHTING_VERTEX_DATA)
    //Combine the tile position into a single vec3
    //NOTE: The level vec4 has been normalised, so scale that up
    vTilePosition = vec3(floor((aVertexColourUnwhitenedRGB_TilePositionLevel.w * 255.0) + 0.1), aMaterialSettingsSlotXY_TilePositionXZ.zw);
#endif
    // Need to add a small epsilon due to interpolation error
    vMaterialSettingsSlotXY_BatchFlags.z = aVertexNormal_BatchFlags.w + (0.25 / 128.0);
//Texture atlas data
#if defined(TEXTURE_ATLAS)
    AssignTextureAtlasVaryings(vTextureUV, vMaterialSettingsSlotXY_BatchFlags.xy, aTextureUV, aMaterialSettingsSlotXY_TilePositionXZ.xy);
#endif

#if defined(DEBUG_VERTEX_BONE_COLOUR)
    #if defined(ANIMATION_VERTEX)
        // We want a pseudo-random colour for each bone (boneID being between 0 and 56).
        vVertexAlbedo.rgba = vec4(mod(aVertexPosition_BoneLabel.w / 5.0, 1.0), mod(aVertexPosition_BoneLabel.w / 14.0, 1.0), mod(aVertexPosition_BoneLabel.w / 63.0, 1.0), 1.0);
    #else
        vVertexAlbedo.rgba = vec4(0.0);
    #endif
#elif defined(MODEL_GEOMETRY_DEPTH_BIAS) && defined(DEBUG_VERTEX_DEPTH_BIAS)
    // force texture slot 0
    vMaterialSettingsSlotXY_BatchFlags.xyz = vec3(0);
    // Supported vis range is from -2/128 to 12/128, red to purple
    // Note this intentionally preserves face alpha
    vVertexAlbedo.rgb = convertHSLtoRGB(vec4(((2.0/128.0) + aVertexTangent.x)*8.0,1.0,0.5,1.0)).rgb;
#endif

#if defined(LIGHT_SCATTERING) || defined(FOG_DISTANCE)
    vec4 worldPosition = uModelMatrix * vec4(aVertexPosition_BoneLabel.xyz, 1.0);
    vec3 viewDirection = worldPosition.xyz - uCameraPosition;
    float cameraDistance = length(viewDirection);

    #if defined(LIGHT_SCATTERING) && defined(SUNLIGHT_DIRECT_LIGHTING)
        ComputeInOutScattering(normalize(viewDirection), cameraDistance, uInvSunDirection.xyz, vOutScattering, vInScattering);
    #else
        vOutScattering = vec3(1.0);
        vInScattering = vec3(0.0);
    #endif

    #if defined(FOG_DISTANCE)
        float fogFactor = FogBasedOnDistance(cameraDistance);
        vInScattering = mix(vInScattering, uFogColour.rgb, fogFactor);
        vOutScattering *= 1.0 - fogFactor;
    #endif
#endif

}
