#include "ConversionUtils.inc"
#include "Shared/ModelGeometryUtils.inc"
#include "Shared/StippleTransparencyUtils.inc"

#if defined(TEXTURE_ATLAS) && defined(ALPHA_ENABLED)
    #include "Shared/ModelGeometryFragmentUtils.inc"
    #include "Shared/NormalMap.inc"
    #include "Shared/TextureCompression.inc"
#endif

void main()
{
#if defined(VERTEX_COLOUR) && defined(ALPHA_ENABLED)
    // Vertex alpha test (as early as possible to avoid unnecessary shader instructions being executed)
    if (vVertexAlbedo.a == 0.0)
    {
        discard;
        return;
    }
#endif

#if defined(ALPHA_ENABLED)
    float alpha = 1.0;

    #if !defined(TEXTURE_ALBEDO_GLOBAL)
        // Check forced texture flag if textures are off globally
        if (fract(vMaterialSettingsSlotXY_BatchFlags.z * 8.0) > 0.5)
        {
    #endif
        #if defined(TEXTURE_ATLAS)
            // Sample the albedo texture to obtain its alpha
            highp vec3 viewVector = vWorldPosition.xyz - uCameraPosition;

            highp vec2 uvDX = vec2(0.0);
            highp vec2 uvDY = vec2(0.0);
            highp vec3 positionDX = vec3(0.0);
            highp vec3 positionDY = vec3(0.0);
            vec4 texelAlbedo = vec4(1.0);

            TextureSettings textureSettings;
            getTextureSettings(vMaterialSettingsSlotXY_BatchFlags.xy, textureSettings);

            // Copy to local highp to workaround precision bug
            highp float textureAnimationTime = uTextureAnimationTime;
            vec2 textureUV = vTextureUV + fract(textureSettings.uvAnim * textureAnimationTime);

            CalculateDerivatives(uvDX, uvDY, positionDX, positionDY, vTextureUV, viewVector);
            getTexel(textureUV, textureSettings.textureMeta1, uAtlasMeta, uvDX, uvDY, textureSettings.wrapping, uTextureAtlas, texelAlbedo);

            // x = Specular, y = Env map
            vec2 alphaUsage = step(0.5, fract(vMaterialSettingsSlotXY_BatchFlags.z * vec2(64.0, 16.0)));
            // Replace alpha if this is a specular or env map texture or if alpha is opaque
            alpha *= min(texelAlbedo.a + alphaUsage.x + alphaUsage.y, 1.0);
        #endif
    #if !defined(TEXTURE_ALBEDO_GLOBAL)
        }
    #endif

    #if defined(VERTEX_COLOUR)
        alpha *= vVertexAlbedo.a;
    #endif

    //Throw away any fragments which are below the set threshold
    if (alpha <= uAlphaTestThreshold)
    {
        discard;
        return;
    }
#endif

    gl_FragColor = uTint;
}
