#include "ConversionUtils.inc"
#include "Shared/ModelGeometryUtils.inc"
#include "Shared/ModelGeometryFragmentUtils.inc"
#include "Shared/NormalMap.inc"
#include "Shared/TextureCompression.inc"
#include "Shared/StippleTransparencyUtils.inc"
#include "Shared/StippleCutoutUtils.inc"

void main()
{
    vec3 normal = normalize(vNormal);

#if defined(TEXTURE_ATLAS) && (defined(ALPHA_ENABLED) || defined(USE_NORMAL_MAP))
    highp vec3 viewVector = vWorldPosition.xyz - uCameraPosition;

    highp vec2 uvDX = vec2(0.0);
    highp vec2 uvDY = vec2(0.0);
    highp vec3 positionDX = vec3(0.0);
    highp vec3 positionDY = vec3(0.0);
    vec4 texelAlbedo = vec4(1.0);
    vec4 texelEmissiveNormal;
    #if defined(COMPRESSED_NORMALS)
        texelEmissiveNormal = vec4(0.0, 0.5, 0.0, 0.5); // 0 emissive (R), no-op normal (GBA = Y, Z (Unused), X)
    #else
        texelEmissiveNormal = vec4(0.0, 0.5, 1.0, 0.5); // 0 emissive (R), no-op normal (GBA = Y, Z, X)
    #endif
    //NOTE: texelEmissiveNormal's normal value will be replaced if we read from a texture, and must be decoded with UnpackNormal further down
    vec4 texelMetalnessRoughness = vec4(0.0); //0 metalness (R) and roughness (G)

    TextureSettings textureSettings;
    getTextureSettings(vMaterialSettingsSlotXY_BatchFlags.xy, textureSettings);

    // Copy to local highp to workaround precision bug
    highp float textureAnimationTime = uTextureAnimationTime;
    vec2 textureUV = vTextureUV + fract(textureSettings.uvAnim * textureAnimationTime);

    CalculateDerivatives(uvDX, uvDY, positionDX, positionDY, vTextureUV, viewVector);
    SampleTexturesRT7(texelAlbedo, texelEmissiveNormal, texelMetalnessRoughness, textureUV, textureSettings, vMaterialSettingsSlotXY_BatchFlags.z, uvDX, uvDY);

    #if defined(ALPHA_ENABLED)
        float alpha = texelAlbedo.a;

        #if defined(VERTEX_COLOUR)
            alpha *= vVertexAlbedo.a;
        #endif

        //Throw away any fragments which are below the set threshold
        if (alpha <= uAlphaTestThreshold)
        {
            discard;
            return;
        }
    #endif

#if defined(STIPPLE_TRANSPARENCY_CUTOUT) && defined(VIEWPORTLOOKUPSCALE)
    if (IsStippleCutoutVisible(viewVector, gl_FragCoord.xy * uViewportLookupScale.xy, gl_FragCoord.xy))
    {
        discard;
        return;
    }
#endif

#if defined(STIPPLE_TRANSPARENCY_ENABLED)
    {
        float srcAlpha;

#if defined(ALPHA_ENABLED)
        srcAlpha = alpha;
#else
        srcAlpha = texelAlbedo.a;
#endif

        if (!IsStipplePixelVisible(viewVector, gl_FragCoord.xy, srcAlpha))
        {
            discard;
            return;
        }
    }
#endif

    #if defined(USE_NORMAL_MAP)
        vec3 normalMap = UnpackNormal(texelEmissiveNormal.gba, textureSettings.normalScale);
        #if defined(PER_FRAGMENT_TANGENTS)
            normal = ApplyNormalMap(normalMap, normal, positionDX, positionDY, uvDX, uvDY);
        #else
            normal = ApplyNormalMap(normalMap, normal, vTangent);
        #endif
        normal = normalize(Mat4ToMat3(uViewMatrix) * normal);
    #endif
#endif

    gl_FragColor = vec4(normal * 0.5 + 0.5, 1.0);
}
