uniform sampler2D sSourceTex;
uniform vec4 uOutlineAndShadowRcpFrame;
uniform vec4 uOutlineAndShadowColour;
uniform bool uOutlineAndShadowSwitch;
uniform bool uOutlineAndShadowClampSwitch;

in vec2 vTexCoord;

bool visible(vec4 rgba)
{
    return rgba.a > 0.02; // Add a threshold of visibility to cut out DXT-compression artifacts (square black borders) on icon models
}

void main()
{
    vec4 source = texture2D(sSourceTex, vTexCoord);

    if (uOutlineAndShadowClampSwitch) //Refer to ObjType.java //Add fake opacity.  Runs after normal outlining.
    {
        if (!visible(source))
        {
            gl_FragColor = vec4(0.0);
        }
        else
        {
            gl_FragColor = vec4(source.rgb, 1.0); // We don't want to leave any alpha values that are between 0 and 1 - so if we have any alpha, make it full alpha
        }
        return;
    }

    if (visible(source))
    {
        gl_FragColor = source;
        return;
    }

    vec4 outline = texture2D(sSourceTex, vTexCoord + uOutlineAndShadowRcpFrame.xy * vec2(-1.0, 0.0)); //Left
    outline += texture2D(sSourceTex, vTexCoord + uOutlineAndShadowRcpFrame.xy * vec2(1.0,  0.0)); //Right
    outline += texture2D(sSourceTex, vTexCoord + uOutlineAndShadowRcpFrame.xy * vec2(0.0,  1.0)); //Top
    outline += texture2D(sSourceTex, vTexCoord + uOutlineAndShadowRcpFrame.xy * vec2(0.0, -1.0)); //Bottom
    vec4 shadow = texture2D(sSourceTex, vTexCoord + uOutlineAndShadowRcpFrame.xy * vec2(-1.0, 1.0)); //Top Left
    // We do both texture lookups and then select the correct result because DX9/Angle
    if ((uOutlineAndShadowSwitch && !visible(shadow)) || !visible(outline))
    {
        gl_FragColor = vec4(0.0);
        return;
    }
    gl_FragColor = uOutlineAndShadowColour;
}
