#ifndef PACK_UTILS_INC
#define PACK_UTILS_INC

#include "Common.inc"


/** Pack a 32bit float into an integer RGBA */
//[ //The obfuscator doesn't like functions with multiple returns
vec4 PackFloatToRGBA(highp float valueToPack)
{
#if defined(USE_MOD_PACK) || defined(USE_FRACT_PACK)
    //const vec4 bitShift = vec4(256 * 256 * 256, 256 * 256, 256, 1.0);
    //const vec4 bitMask = vec4(0, 1.0 / 256.0, 1.0 / 256.0, 1.0 / 256.0);
    //vec4 fragColour = fract(valueToPack * bitShift);
    //return fragColour - (fragColour.xxyz * bitMask);

    const highp vec4 bitShift = vec4(256 * 256 * 256, 256 * 256, 256, 1.0);
    const highp vec4 bitMask = vec4(0, 1.0 / 256.0, 1.0 / 256.0, 1.0 / 256.0);
    // combination of mod and multiplication and division works better
    highp vec4 fragColour = mod(valueToPack * bitShift * vec4(255), vec4(256)) / vec4(255);
    return fragColour - fragColour.xxyz * bitMask;
#endif

#ifdef USE_ARAS_PACK
    const highp vec4 bitShift = vec4(1.0, 255.0, 65025.0, 16581375.0);
    const highp vec4 bitMask = vec4(1.0 / 255.0, 1.0 / 255.0, 1.0 / 255.0, 0.0);
    highp vec4 fragColour = fract(valueToPack * bitShift);
    return fragColour - (fragColour.xxyz * bitMask);
#endif
}
//]

/** Pack normal to vec2<uint16> "Spherical environment mapping" */
vec2 NormalPackSphereMap(vec3 valueToPack)
{
    vec2 enc = normalize(valueToPack.xy) * (sqrt(-valueToPack.z * 0.5 + 0.5));
    enc = enc * 0.5 + 0.5;
    return enc * 65535.0;
}

/** Pack 16 bit float to colour vec2 */
vec2 PackFloatToVec2(float valueToPack)
{
    vec2 result;
    const float inv255 = 1.0 / 255.0;
    vec2 multiplier = vec2(1.0, 255.0);
    vec2 enc = fract(multiplier * valueToPack);
    enc.x -= enc.y * inv255;
    return enc.xy;
}


#endif
