#include "ParticleCommon.inc"
#include "ParticleLighting.inc"
#include "Shared/ViewportMap/ViewportMap.inc"

//Main function
void main()
{
    // Get vertex in camera space
    vWorldPosition = vec4(aParticleOrigin_CreationTime.xyz, 1.0);

    // Need to disable the minifier for this as it gets confused, presumably due to the TestParticleFlag use - see GT-4160
    //[
    vec4 cameraSpacePosition;

    if (TestParticleFlag(int(aParticleUVAnimationData.z), PARTICLE_FLAG_HEADING_ALIGNED_BIT))
    {
        vec3 wsOriginDir = normalize(uCameraPosition.xyz - vWorldPosition.xyz);
        ApplyParticleOffset3DWS(vWorldPosition, aParticleVelocity, aParticleOffset, wsOriginDir);
        cameraSpacePosition = uViewMatrix * vWorldPosition;
    }
    else
    {
        cameraSpacePosition = uViewMatrix * vWorldPosition;
        ApplyParticleOffset2DVS(cameraSpacePosition, aParticleOffset, aMaterialSettingsSlotXY_Rotation.zw);
        vWorldPosition = uInvViewMatrix * cameraSpacePosition;
    }
    //]

    vec3 viewDirection = vWorldPosition.xyz - uCameraPosition;
    float cameraDistance = length(viewDirection);
    viewDirection /= cameraDistance;

#if defined(LIGHT_SCATTERING) || defined(FOG_DISTANCE)
    #if defined(LIGHT_SCATTERING) && defined(SUNLIGHT_DIRECT_LIGHTING)
        ComputeInOutScattering(viewDirection, cameraDistance, uInvSunDirection, vOutScattering, vInScattering);
    #else
        vOutScattering = vec3(1.0);
        vInScattering = vec3(0.0);
    #endif

    #if defined(FOG_DISTANCE)
        float fogFactor = FogBasedOnDistance(cameraDistance);
        vInScattering = mix(vInScattering, uFogColour.rgb, fogFactor);
        vOutScattering *= 1.0 - fogFactor;
    #endif
#endif

    vec4 uv;
    float animationT;
    CalculateParticleUV(uv, animationT, aParticleOffset, aParticleOrigin_CreationTime.w, aParticleUVAnimationData);

    AssignParticleVaryings(aVertexColour, uv, animationT, aMaterialSettingsSlotXY_Rotation.xy);

    // Only output any alpha if we're using non-additive blending as our rendermode
    // Need to disable the minifier for this as it gets confused, presumably due to the TestParticleFlag use - see GT-4160
    //[
    vAlphaOutput = 1.0 - min(1.0, float(TestParticleFlag(int(aParticleUVAnimationData.z), PARTICLE_FLAG_ADDITIVE_RENDER_MODE_BIT)));
    //]

    // Output linear view space depth
#if defined(SOFT_PARTICLES) || defined(LIT_PARTICLES)
    vViewSpaceDepth = cameraSpacePosition.z;
#endif

#if defined(SOFT_PARTICLES)
    // Storing this flag as a float here so we don't have to check it for each pixel instead
    vSoftFadeEnabled = 1.0 - float(TestParticleFlag(int(aParticleUVAnimationData.z), PARTICLE_FLAG_NO_SOFT_FADE));
#endif

#if defined(LIT_PARTICLES) && defined(SUNLIGHT_DIRECT_LIGHTING) && defined(SUNLIGHT_SHADOWS)
    vLightingData = mat3(0.0);

    // Only apply lighting/shadows to alpha-blended particles
    if (vAlphaOutput > 0.0)
    {
        CalculateParticleVertexLighting(vec4(aParticleOrigin_CreationTime.xyz, 1.0), vWorldPosition, vViewSpaceDepth, aParticleUVAnimationData.z, aParticleUVAnimationData.w);
    }
    else
    {
        vLightingContribution = 0.0;
        vLightingShadowStrength = 0.0;
    }
#endif

    // Compute final clip space position
    gl_Position = uProjectionMatrix * cameraSpacePosition;
}
