#ifndef PARTICLE_LIGHTING_INC
#define PARTICLE_LIGHTING_INC

#if defined(SUNLIGHT_DIRECT_LIGHTING) && defined(SUNLIGHT_SHADOWS)

#include "ParticleCommon.inc"
#include "HL2Basis.inc"
#include "Shared/LightingUtils.inc"
#include "lighting/Lighting.inc"

float EvaluateParticleSunlight(vec4 worldPosition, float viewSpaceDepth, float shadowStrength)
{
    int iCascadeInOut = 0;
    highp vec4 sunlightViewPos = uSunlightViewMatrix * worldPosition;
    float amountInSunlight = DirLightShadowAtten(iCascadeInOut, worldPosition, sunlightViewPos, viewSpaceDepth, uSunlightShadowMap, uSunlightShadowTranslucencyMap, 1.0);

    return mix(1.0, amountInSunlight, shadowStrength);
}

#if defined(SHADER_TYPE_PIXEL)
in mat3 vLightingData;
flat in float vLightingFlags;
flat in float vLightingContribution;
flat in float vLightingShadowStrength;

vec3 ApplyParticleLighting(vec3 fragColour, vec3 worldNormal, vec4 worldPosition, float viewSpaceDepth)
{
    vec3 lighting;

    if (TestParticleFlag(int(vLightingFlags), PARTICLE_FLAG_LIGHTING_PER_PIXEL))
    {
        float amountInSunlight = EvaluateParticleSunlight(worldPosition, viewSpaceDepth, vLightingShadowStrength);
        float contribution = clamp(dot(worldNormal, uInvSunDirection), 0.0, 1.0) * amountInSunlight;
        lighting = contribution * uSunColour;
    }
    else
    {
        // Per particle/vertex lighting
        lighting = CalculateHL2BasisLighting(vLightingData, worldNormal);
    }

    return mix(fragColour.rgb, fragColour.rgb * lighting, vLightingContribution);
}
#else
out mat3 vLightingData;
flat out float vLightingFlags;
flat out float vLightingContribution;
flat out float vLightingShadowStrength;

void CalculateParticleVertexLighting(vec4 particleWorldPosition, vec4 worldPosition, float viewSpaceDepth, float flags, float lightingData)
{
    vLightingFlags = flags;

    UnpackFloat8(vLightingContribution, vLightingShadowStrength, lightingData);

    // Convert back from a 0..16 range to 0..1
    vLightingShadowStrength /= 16.0;
    vLightingContribution /= 16.0;

    if ((vLightingContribution != 0.0) && !TestParticleFlag(int(flags), PARTICLE_FLAG_LIGHTING_PER_PIXEL))
    {
        vec4 usedWorldPosition;
        float usedViewSpaceDepth;

        if (TestParticleFlag(int(flags), PARTICLE_FLAG_LIGHTING_PER_VERTEX))
        {
            // Per vertex
            usedWorldPosition = worldPosition;
            usedViewSpaceDepth = viewSpaceDepth;
        }
        else
        {
            // Per particle
            usedWorldPosition = particleWorldPosition;
            usedViewSpaceDepth = (uViewMatrix * usedWorldPosition).z;
        }

        float amountInSunlight = EvaluateParticleSunlight(usedWorldPosition, usedViewSpaceDepth, vLightingShadowStrength);

        AccumulateHL2BasisLighting(vLightingData, uInvSunDirection, uSunColour, amountInSunlight);
    }
}
#endif

#endif

#endif
