#include "Shared/DepthUtils.inc"

uniform mat4 uSceneProjectionMatrix;
uniform mat4 uSceneInvProjMatrix;
uniform sampler2D uNoiseTexture;
uniform SAMPLER_2D_AUTO_MULTISAMPLE uNormalTexture;
uniform SAMPLER_2D_AUTO_MULTISAMPLE uDepthTexture;
uniform vec4 uParams;       // x=radius, y=sigma, z=bias, w=projectionFactor x radius
uniform vec2 uInverseNoiseTexSize;
uniform float uZFar;
uniform float uMinSamples;
uniform float uBaseSamples;

in vec2 vTextureUV;

void main()
{
    vec2 screenTexCoords = vTextureUV * uViewportOffsetScale.zw + uViewportOffsetScale.xy;
    vec3 wsNormal = texture2DMultisample(uNormalTexture, screenTexCoords, uFullScreenLookupScale.zw).xyz;
    // Skip sky pixels
    if (dot(wsNormal, vec3(1.0)) == 0.0)
    {
        gl_FragColor = vec4(1.0);
        return;
    }
    wsNormal = wsNormal * 2.0 - 1.0;
    vec3 wsPosition = GetViewSpacePos(vTextureUV, texture2DMultisample(uDepthTexture, screenTexCoords, uFullScreenLookupScale.zw).r, uSceneInvProjMatrix);

    float ambient_occlusion = 0.0;
    float projected_radius = uParams.w / wsPosition.z;
    const float epsilon = 0.00001;
    float samples = max((uBaseSamples * uZFar) / ( uZFar + uBaseSamples * wsPosition.z ), uMinSamples);
    int sample_count = int(min(uBaseSamples, samples));
    vec2 sample_offset = gl_FragCoord.xy * uInverseNoiseTexSize;
    for (int i = 0; i < sample_count; ++i)
    {
        float sample_x = float(i) * uInverseNoiseTexSize.x;
        vec2 sample_random_direction = texture2DLodCompat(uNoiseTexture, vec2(sample_x, sample_x * uInverseNoiseTexSize.y) + sample_offset, 0.0).xy;
        sample_random_direction = normalize(sample_random_direction * 2.0 - 1.0);

        vec3 tc_sample;
        tc_sample.xy = vTextureUV + sample_random_direction * projected_radius * uViewportLookupScale.xy;
        vec2 sampleUV = tc_sample.xy * uViewportOffsetScale.zw + uViewportOffsetScale.xy;
        tc_sample.z = texture2DMultisample(uDepthTexture, sampleUV, uFullScreenLookupScale.zw).r;
        tc_sample = tc_sample * 2.0 - 1.0;
        vec4 wsSample = uSceneInvProjMatrix * vec4(tc_sample, 1.0);
        wsSample.xyz /= wsSample.w;

        vec3 v = wsSample.xyz - wsPosition.xyz;

        ambient_occlusion += max(0.0, dot(v, wsNormal) - wsSample.z * uParams.z) / (dot(v, v) + epsilon);
    }

    ambient_occlusion = max(0.0, 1.0 - 2.0 * uParams.y / float(samples) * ambient_occlusion);
    gl_FragColor = vec4(ambient_occlusion, ambient_occlusion, ambient_occlusion, 1.0);
}
