#ifndef SHADOW_MAPPING_INC
#define SHADOW_MAPPING_INC


#define MAX_ATLAS_CASCADES 4


/**
    Makes the necessary adjustments to the clip space z position when rendering into a shadow depth buffer.
    This should always be called last in all shadow vertex shaders.
 */
void AdjustShadowClipSpaceZPos(inout float shadowClipSpacePosZOut)
{
#if defined(OGLES2_BACKEND) || defined(OGLES3_BACKEND) || defined(OGL_BACKEND)
    // Expand back out to -1-1 range to ensure that we're 0-1 range in the depth buffer,
    // as OpenGL's viewport transform math (after clipping and division by w) always assumes that
    // incoming clip space z values are in the range -1-1
    shadowClipSpacePosZOut = (shadowClipSpacePosZOut - 0.5) * 2.0;
#endif

    //Clamp Z to to near clip (a.ka. pancaking) so that the rasterizer does not discard
    //the casters behind the near clip plane in shadow light view space.
    const float shadowProjMatNearClip = -1.0; // TODO: Should ideally be set as a constant by application
    shadowClipSpacePosZOut = max(shadowProjMatNearClip, shadowClipSpacePosZOut);
}


#endif
