#include "UndefineInOut.inc"
#include "lighting/ShadowUtils.inc"

#if defined(NXT_MOBILE)
    #define USE_LIGHT_VIEW_PROJ_TEX_MATRIX 1 // Must be used in conjunction with LightProjectionFit::BOUNDING_SPHERE_FIT

    #if defined(SHADOWS_QUALITY_LOW)
        #define SHADOW_FADE_STANDARD 1
        #define CASCADE_SPLIT_SELECTION 1
        #define SHADOW_MAP_FILTER_BOX2X2 1
        #define SHADOW_MAP_TRANSLUCENCY_FILTER_BOX2X2 1
    #elif defined(SHADOWS_QUALITY_MED)
        #define SHADOW_FADE_STANDARD 1
        #define CASCADE_SPLIT_SELECTION 1
        #define SHADOW_MAP_FILTER_BOX2X2 1
        #define SHADOW_MAP_TRANSLUCENCY_FILTER_BOX2X2 1
    #elif defined(SHADOWS_QUALITY_HIGH)
        #define SHADOW_FADE_SMOOTH 1
        #define CASCADE_BOUNDING_SPHERE_SELECTION 1
        #define SHADOW_MAP_FILTER_BOX4X4_APPROX 1
        #define SHADOW_MAP_TRANSLUCENCY_FILTER_BOX4x4_APPROX 1
    #elif defined(SHADOWS_QUALITY_ULTRA)
        #define SHADOW_FADE_SMOOTH 1
        #define CASCADE_BOUNDING_SPHERE_SELECTION 1
        #define SHADOW_MAP_FILTER_BOX4X4 1
        #define SHADOW_MAP_TRANSLUCENCY_FILTER_BOX4x4 1
    #else
        #error Sunlight shadow quality not defined
    #endif
#else
    #if defined(SHADOWS_QUALITY_LOW)
        #define USE_LIGHT_VIEW_PROJ_TEX_MATRIX 1 // Must be used in conjunction with LightProjectionFit::BOUNDING_SPHERE_FIT
        #define SHADOW_FADE_STANDARD 1
        #define CASCADE_BOUNDING_SPHERE_SELECTION 1
        #define SHADOW_MAP_FILTER_BOX2X2 1
        #define SHADOW_MAP_TRANSLUCENCY_FILTER_BOX2X2 1
    #elif defined(SHADOWS_QUALITY_MED)
        #define USE_LIGHT_VIEW_PROJ_TEX_MATRIX 1 // Must be used in conjunction with LightProjectionFit::BOUNDING_SPHERE_FIT
        #define SHADOW_FADE_STANDARD 1
        #define CASCADE_BOUNDING_SPHERE_SELECTION 1
        #define SHADOW_MAP_FILTER_BOX4X4_APPROX 1
        #define SHADOW_MAP_TRANSLUCENCY_FILTER_BOX2X2 1
    #elif defined(SHADOWS_QUALITY_HIGH)
        #define SHADOW_FADE_SMOOTH 1
        #define CASCADE_BEST_MAP_SELECTION 1
        #define SHADOW_MAP_FILTER_BOX4X4 1
        #define SHADOW_MAP_TRANSLUCENCY_FILTER_BOX4x4_APPROX 1
    #elif defined(SHADOWS_QUALITY_ULTRA)
        #define SHADOW_FADE_SMOOTH 1
        #define CASCADE_BEST_MAP_SELECTION 1
        #define SHADOW_MAP_FILTER_BOX4X4 1
        #define SHADOW_MAP_TRANSLUCENCY_FILTER_BOX4x4 1
    #else
        #error Sunlight shadow quality not defined
    #endif
#endif

/** Determines which shadow map cascade we fall into using the cascade split plane distances */
int ShadowMapSelectCascadeBySplit(highp float pixelViewDepth, highp vec4 cascadeFrustumViewDepths, vec4 cascadeFlags)
{
    // equivalent to (pixelViewDepth >= cascadeFrustumViewDepths)
    vec4 compare = 1.0 - step(pixelViewDepth, cascadeFrustumViewDepths);
    float iCascade = dot(cascadeFlags, compare);
    return int(iCascade);
}

/**
    Computes the texture coordinates for shadow map sampling according to the first
    atlas quadrant that the input world position falls into. This produces higher
    quality shadow sampling, but at extra cost over split selection due to branch instructions.
    */
int ShadowMapSelectCascadeByMap(
    inout vec4 shadowMapTexCoords,
    const highp vec4 lightViewPos,
    const highp vec4 lightProjTexMatScale[MAX_ATLAS_CASCADES],
    const highp vec4 lightProjTexMatOffset[MAX_ATLAS_CASCADES],
    const vec4 cascadeMinAtlasExtents[MAX_ATLAS_CASCADES])
{
    float iCascade = 0.0;

    shadowMapTexCoords = (lightViewPos * lightProjTexMatScale[0]) + lightProjTexMatOffset[0];
    // Note. As this is an orthographic projection, the divide by w is not required
    //shadowMapTexCoords.xy /= shadowMapTexCoords.w;

    // If texcoords fall outside of the range 0.5<=x<=1 0.0<=y<=0.5then this position is outside the first cascade
    if (min(shadowMapTexCoords.x, shadowMapTexCoords.y) < cascadeMinAtlasExtents[0].x ||
        max(shadowMapTexCoords.x, shadowMapTexCoords.y) > cascadeMinAtlasExtents[0].z ||
        shadowMapTexCoords.z < 0.0 || shadowMapTexCoords.z > 1.0)
    {
        iCascade = 1.0;
        shadowMapTexCoords = (lightViewPos * lightProjTexMatScale[1]) + lightProjTexMatOffset[1];

        // If texcoords fall outside of the range 0.5<=x<=1 0.0<=y<=0.5 then this position is outside the second cascade
        if (shadowMapTexCoords.x < cascadeMinAtlasExtents[1].x || shadowMapTexCoords.x > cascadeMinAtlasExtents[1].z ||
            shadowMapTexCoords.y < cascadeMinAtlasExtents[1].y || shadowMapTexCoords.y > cascadeMinAtlasExtents[1].w ||
            shadowMapTexCoords.z < 0.0 || shadowMapTexCoords.z > 1.0)
        {
            iCascade = 2.0;
            shadowMapTexCoords = (lightViewPos * lightProjTexMatScale[2]) + lightProjTexMatOffset[2];

            // If texcoords fall outside of the range 0.0<=x<=0.5 0.0<=y<=0.5 then this position is outside the second cascade
            if (shadowMapTexCoords.x < cascadeMinAtlasExtents[2].x || shadowMapTexCoords.x > cascadeMinAtlasExtents[2].z ||
                shadowMapTexCoords.y < cascadeMinAtlasExtents[2].y || shadowMapTexCoords.y > cascadeMinAtlasExtents[2].w ||
                shadowMapTexCoords.z < 0.0 || shadowMapTexCoords.z > 1.0)
            {
                iCascade = 3.0;
                shadowMapTexCoords = (lightViewPos * lightProjTexMatScale[3]) + lightProjTexMatOffset[3];

                // If texcoords fall outside of the range 0.5<=x<=1 0.5<=y<=1.0 then this position is outside the second cascade
                if (min(shadowMapTexCoords.x, shadowMapTexCoords.y) < cascadeMinAtlasExtents[3].x ||
                    max(shadowMapTexCoords.x, shadowMapTexCoords.y) > cascadeMinAtlasExtents[3].z ||
                    shadowMapTexCoords.z < 0.0 || shadowMapTexCoords.z > 1.0)
                {
                    iCascade = 4.0;
                }
            }
        }
    }

    return int(iCascade);
}

int ShadowMapSelectCascadeByMap(
    inout vec4 shadowMapTexCoords,
    const highp vec4 worldPos,
    const highp mat4 lightViewProjTexMat[MAX_ATLAS_CASCADES],
    const vec4 cascadeMinAtlasExtents[MAX_ATLAS_CASCADES])
{
    float iCascade = 0.0;

    shadowMapTexCoords = lightViewProjTexMat[0] * worldPos;

    // If texcoords fall outside of the range 0.5<=x<=1 0.0<=y<=0.5then this position is outside the first cascade
    if (min(shadowMapTexCoords.x, shadowMapTexCoords.y) < cascadeMinAtlasExtents[0].x ||
        max(shadowMapTexCoords.x, shadowMapTexCoords.y) > cascadeMinAtlasExtents[0].z ||
        shadowMapTexCoords.z < 0.0 || shadowMapTexCoords.z > 1.0)
    {
        iCascade = 1.0;
        shadowMapTexCoords = lightViewProjTexMat[1] * worldPos;

        // If texcoords fall outside of the range 0.5<=x<=1 0.0<=y<=0.5 then this position is outside the second cascade
        if (shadowMapTexCoords.x < cascadeMinAtlasExtents[1].x || shadowMapTexCoords.x > cascadeMinAtlasExtents[1].z ||
            shadowMapTexCoords.y < cascadeMinAtlasExtents[1].y || shadowMapTexCoords.y > cascadeMinAtlasExtents[1].w ||
            shadowMapTexCoords.z < 0.0 || shadowMapTexCoords.z > 1.0)
        {
            iCascade = 2.0;
            shadowMapTexCoords = lightViewProjTexMat[2] * worldPos;

            // If texcoords fall outside of the range 0.0<=x<=0.5 0.0<=y<=0.5 then this position is outside the second cascade
            if (shadowMapTexCoords.x < cascadeMinAtlasExtents[2].x || shadowMapTexCoords.x > cascadeMinAtlasExtents[2].z ||
                shadowMapTexCoords.y < cascadeMinAtlasExtents[2].y || shadowMapTexCoords.y > cascadeMinAtlasExtents[2].w ||
                shadowMapTexCoords.z < 0.0 || shadowMapTexCoords.z > 1.0)
            {
                iCascade = 3.0;
                shadowMapTexCoords = lightViewProjTexMat[3] * worldPos;

                // If texcoords fall outside of the range 0.5<=x<=1 0.5<=y<=1.0 then this position is outside the second cascade
                if (min(shadowMapTexCoords.x, shadowMapTexCoords.y) < cascadeMinAtlasExtents[3].x ||
                    max(shadowMapTexCoords.x, shadowMapTexCoords.y) > cascadeMinAtlasExtents[3].z ||
                    shadowMapTexCoords.z < 0.0 || shadowMapTexCoords.z > 1.0)
                {
                    iCascade = 4.0;
                }
            }
        }
    }

    return int(iCascade);
}

/**
    Can be used as a quicker alternative to ShadowMapSelectCascadeByMap() when using the bounding sphere projection fit
    */
int ShadowMapSelectCascadeBySphere(
    const highp vec4 worldPos,
    const highp vec4 cascadeWorldBoundingSphere[MAX_ATLAS_CASCADES])
{
    float iCascade = 0.0;

    // Calculate distance between worldPos and cascade bounding sphere centre pos
    // Each component contains the equivalent of (worldPos - spherePos).LengthSq()
    highp vec4 distances = vec4(
        dot(worldPos.xyz - cascadeWorldBoundingSphere[0].xyz, worldPos.xyz - cascadeWorldBoundingSphere[0].xyz),
        dot(worldPos.xyz - cascadeWorldBoundingSphere[1].xyz, worldPos.xyz - cascadeWorldBoundingSphere[1].xyz),
        dot(worldPos.xyz - cascadeWorldBoundingSphere[2].xyz, worldPos.xyz - cascadeWorldBoundingSphere[2].xyz),
        dot(worldPos.xyz - cascadeWorldBoundingSphere[3].xyz, worldPos.xyz - cascadeWorldBoundingSphere[3].xyz));

    // Determine which cascade the worldPos fall into by performing a distance/radius check
    // Note that we require the radius squared to be contained in the w component of the cascade bounding sphere uploaded
    iCascade = distances.x < cascadeWorldBoundingSphere[0].w ? 0.0 :
        distances.y < cascadeWorldBoundingSphere[1].w ? 1.0 :
        distances.z < cascadeWorldBoundingSphere[2].w ? 2.0 :
        distances.w < cascadeWorldBoundingSphere[3].w ? 3.0 : 4.0;

    return int(iCascade);
}

/** @return Directional light shadow attenuation (i.e. amount in light) */
float DirLightShadowAtten(
    inout int iCascade,
    const highp vec4 worldPos,
    const highp vec4 lightViewPos,
    const highp float pixelViewDepth,
    const SHADOW_SAMPLER shadowDepthMapAtlas,
    const sampler2D shadowTranslucencyMapAtlas,
    float shadowStrengthMul)
{
    highp vec4 shadowMapTexCoords;

    // Select cascade
    int cascadeCount = int(uMappingParams.w);
    #if defined(CASCADE_SPLIT_SELECTION)
        iCascade = ShadowMapSelectCascadeBySplit(pixelViewDepth, uCascadeFrustumViewDepths, uCascadeSplitSelectionFlags);
    #elif defined(CASCADE_BOUNDING_SPHERE_SELECTION)
        iCascade = ShadowMapSelectCascadeBySphere(worldPos, uCascadeWorldBoundingSpheres);
    #elif defined(CASCADE_BEST_MAP_SELECTION)
        iCascade = ShadowMapSelectCascadeByMap(shadowMapTexCoords, lightViewPos, uSunlightProjTexMatScale, uSunlightProjTexMatOffset, uCascadeMinAtlasExtents);
    #endif

    // Perform light attenuation
    float amountInLight = 1.0;
    if (iCascade < cascadeCount)
    {
        vec4 shadowFilterUVExtents;

    #if defined(CASCADE_SPLIT_SELECTION) || defined(CASCADE_BOUNDING_SPHERE_SELECTION)
        // Given the map index, transform into shadow map texture space.
        // We do this after the conditional to save cycles if pixel is beyond shadow mapping range
        #if defined(USE_LIGHT_VIEW_PROJ_TEX_MATRIX)
            shadowMapTexCoords = uSunlightViewProjTexMatrix[iCascade] * worldPos;
        #else
            shadowMapTexCoords = (lightViewPos * uSunlightProjTexMatScale[iCascade]) + uSunlightProjTexMatOffset[iCascade];
        #endif
        //shadowMapTexCoords.xy /= shadowMapTexCoords.w; // Not actually needed for orthographic projection from sunlight

        shadowFilterUVExtents = uCascadeMaxAtlasExtents[iCascade];
    #else
        shadowFilterUVExtents = uCascadeMinAtlasExtents[iCascade];
    #endif

        // Perform opaque object shadow filtering
    #if !defined(SHADOWS_QUALITY_LOW)
        if (pixelViewDepth < uLowShadowQualityDepth)
        {
        #if defined(SHADOW_MAP_FILTER_BOX2X2)
            amountInLight = ShadowDepthMapFilter2x2Box(shadowDepthMapAtlas, shadowMapTexCoords, uMappingParams.xx, shadowFilterUVExtents);
        #elif defined(SHADOW_MAP_FILTER_BOX4X4_APPROX)
            amountInLight = ShadowDepthMapFilter4x4BoxApprox(shadowDepthMapAtlas, shadowMapTexCoords, uMappingParams.xx, shadowFilterUVExtents);
        #elif defined(SHADOW_MAP_FILTER_BOX4X4)
            amountInLight = ShadowDepthMapFilter4x4Box(shadowDepthMapAtlas, shadowMapTexCoords, uMappingParams.xx, shadowFilterUVExtents);
        #elif defined(SHADOW_MAP_FILTER_BOX8X8)
            amountInLight = ShadowDepthMapFilter8x8Box(shadowDepthMapAtlas, shadowMapTexCoords, uMappingParams.xx, shadowFilterUVExtents);
        #elif defined(SHADOW_MAP_FILTER_DISK)
            amountInLight = ShadowDepthMapDiskFilter(shadowDepthMapAtlas, shadowMapTexCoords, uMappingParams.xx, shadowFilterUVExtents);
        #else
            #error No opaque shadow filter defined
        #endif
        }
        else
    #endif
        {
            amountInLight = ShadowDepthMapFilter2x2Box(shadowDepthMapAtlas, shadowMapTexCoords, uMappingParams.xx, shadowFilterUVExtents);
        }

    #if defined(TRANSLUCENT_SHADOWS)
        // We need to use a matching filter kernel when sampling from the translucency map. If an object is rendered in front of a particle (from the viewpoint of the light),
        // occluded particle fragments will mean nothing is rendered into the translucency map at those pixel locations indicating full translucency. This causes white silhouette edges
        // to be formed at the edges of opaque objects in penumbra regions when particle shadows overlap. This happens because the shadow filter kernel is sampling a penumbra region of shadow,
        // but the value sampled from the translucency map will be 1.0 (or fully translucent) due to those particle fragments being depth occluded.
        float translucency = 1.0;
        // If we're not in a penumbra region, then we only need to sample the translucency map with a single tap.
        // We could extend this check to include areas that are not in shadow, but this causes artifacts.
        if (amountInLight > 0.0)
        {
        #if defined(SHADOW_MAP_TRANSLUCENCY_FILTER_BOX2X2)
            translucency = ShadowTranslucencyMapFilter2x2Box(shadowTranslucencyMapAtlas, shadowMapTexCoords.xy, uMappingParams.xx);
        #elif defined(SHADOW_MAP_TRANSLUCENCY_FILTER_BOX4x4_APPROX)
            translucency = ShadowTranslucencyMapFilter4x4BoxApprox(shadowTranslucencyMapAtlas, shadowMapTexCoords.xy, uMappingParams.xx, shadowFilterUVExtents);
        #elif defined(SHADOW_MAP_TRANSLUCENCY_FILTER_BOX4x4)
            translucency = ShadowTranslucencyMapFilter4x4Box(shadowTranslucencyMapAtlas, shadowMapTexCoords.xy, uMappingParams.xx, shadowFilterUVExtents);
        #else
            #error No translucency shadow filter defined
        #endif
        }
        // Combine amountInLight with translucency. We take the min value between the two to give us our final amount in light.
        amountInLight = min(amountInLight, translucency);
    #endif

        // Attenuate according to shadow strength
        amountInLight = min(1.0, amountInLight + (1.0 - (uMappingParams.z * shadowStrengthMul)));

        // Fade must come after final attenuation
    #if defined(SHADOW_FADE_STANDARD)
        amountInLight = ShadowFade(amountInLight, pixelViewDepth, uSunlightFadeAttenParams.x, uSunlightFadeAttenParams.z);
    #elif defined(SHADOW_FADE_SMOOTH)
        amountInLight = ShadowFadeSmooth(amountInLight, pixelViewDepth, uSunlightFadeAttenParams.x, uSunlightFadeAttenParams.y);
    #else
        #error No shadow fade function specified
    #endif
    }

    return amountInLight;
}

#include "DefineInOut.inc"
