#ifndef ENV_LIGHTING_UTILS
#define ENV_LIGHTING_UTILS

#include "UndefineInOut.inc"


/**
    Evaluates the indirect specular lighting from a pre-filtered environment map

    @param envMappingParams x: max mip levels, y: max specular roughness (i.e. 8192), z: <spare>, w: <spare>
 */
void EvaluateEnvironmentLightingRT7(
    out vec3 specularOut,
    vec3 worldNormal,
    vec3 viewDirection,
    vec4 envMappingParams,
    float envFresnel,
    float roughness)
{
    // Previous version would end up with a backwards environment mapping, wrong side and not reflected.
    // So now we get the inverse reflection, and spin around the x and y to put the texture the right way up
    vec3 envMapDirection = reflect(-viewDirection, worldNormal);
    envMapDirection.xy = -envMapDirection.xy;

    // Select mip level to use based on surface roughness
    float mipLevel = envMappingParams.x - ((roughness / envMappingParams.y) * envMappingParams.x);
    mipLevel = clamp(mipLevel, 0.0, 1.0);
    // Sample environment map and modify by environment BRDF
    specularOut = textureCubeLodSRGB(uGlobalEnvironmentMap, envMapDirection, mipLevel).rgb;
}


#include "DefineInOut.inc"

#endif
