#ifndef ENV_MAPPING_INC
#define ENV_MAPPING_INC


/**
    Helper for sampling specular lighting from a cubic environment map taking into account surface roughness

    @param specInOut Indirect specular lighting to which the cube map sample to applied to
    @param envMipLevels The number of mip levels in the cube map
    @param envFresnel Fresnel for surface at current fragment which is applied to the cube map sample
    @param invRoughness Inverse of surface roughness (i.e. 0 is completely rough) used to compute correct mip level from cube map
    @param ambientOcclusion Ambient occlusion term which is applied to specInOut
 */
void SampleCubeMapSpecular(
    inout vec3 specInOut,
    float envMipLevels,
    samplerCube envMap,
    vec3 viewDir,
    vec3 worldNormal,
    vec3 envBRDF,
    float invRoughness)
{
    // Previous version would end up with a backwards environment mapping, wrong side and not reflected.
    // So now we get the inverse reflection, and spin around the x and y to put the texture the right way up
    vec3 envMapDirection = reflect(viewDir, worldNormal);
    envMapDirection.xy = -envMapDirection.xy;

    // Select mip level to use based on surface roughness
    float mipLevel = max(0.0, envMipLevels - (invRoughness * envMipLevels));
    // Sample environment map and modify by environment BRDF
    vec3 cubeMap = textureCubeLodSRGB(envMap, envMapDirection, mipLevel).rgb;
    specInOut += cubeMap * envBRDF;
}


#endif
