#ifndef LIGHT_SCATTERING_VS_FUNCTIONS
#define LIGHT_SCATTERING_VS_FUNCTIONS

#include "UndefineInOut.inc"

void ComputeInOutScattering(highp vec3 nViewVector, highp float viewDist, highp vec3 invSunDir, out vec3 extinction, out vec3 inScattering)
{
    float distance = max(0.0, viewDist - uScatteringParameters.y);
    highp float sunAmount = pow(max(0.0, dot(nViewVector, invSunDir)), 2.0);
    vec3 fogColour = mix(uScatteringTintColour, vec3(1.0,1.0,1.0), sunAmount);

    // See LightScatteringEffect.cpp for the reasoning behind this multiply
    float atmosphere = (-distance * uScatteringParameters.x) * 0.001;

    // Rewritten so we inscatter faster towards the sun, but max out at the inscatter colour
    vec3 outscatteringColour = exp(atmosphere * uOutscatteringAmount.rgb);
    vec3 inscatteringColour = uInscatteringAmount * (vec3(1.0) - exp(atmosphere * fogColour));

    extinction = outscatteringColour;
    inScattering = inscatteringColour;
}

#include "DefineInOut.inc"

#endif
