#if defined(ANIMATION_VERTEX)

// Note: I've duplicated the entire GetBoneFromIndex() function inside this if/else condition as having them inside the function
// seems to be messing up the shading minifier causing it to generate incorrect code
#if defined(USE_BONE_HALF_FLOATS)

mat4 GetBoneFromIndex(float boneIndex)
{
    // Every two bone matrices straddles 3 uvec4s. If the index is even we have a straight 3/2 mapping from the bone index to the uvec4 array index,
    // and we access the first six elements. If the index is odd we get the uvec4 after that one and access the last six elements. To avoid using an
    // if statement we use the even/odd mask as a multiplier to select one of the two possibilities.
    uint iBoneIndex = uint(boneIndex);
    uint mask = iBoneIndex % 2u;
    uint uvec4Index = (iBoneIndex * 3u - mask) / 2u;
    uvec4 t1 = (1u-mask) * uBoneTransforms[uvec4Index] + mask * uvec4(uBoneTransforms[uvec4Index].zw, uBoneTransforms[uvec4Index + 1u].xy);
    uvec2 t2 = (1u-mask) * uvec2(uBoneTransforms[uvec4Index + 1u].xy) + mask * uvec2(uBoneTransforms[uvec4Index + 1u].zw);

    // We must unpack the boneTransforms to vec2s
    vec2 v1 = unpackHalf2x16(t1.x);
    vec2 v2 = unpackHalf2x16(t1.y);
    vec2 v3 = unpackHalf2x16(t1.z);
    vec2 v4 = unpackHalf2x16(t1.w);
    vec2 v5 = unpackHalf2x16(t2.x);
    vec2 v6 = unpackHalf2x16(t2.y);

    mat4 boneMatrix = mat4(
        vec4(v1.xy, v2.x, 0.0),
        vec4(v2.y, v3.xy, 0.0),
        vec4(v4.xy, v5.x, 0.0),
        vec4(v5.y, v6.xy, 1.0));
    return boneMatrix;
}

#else

mat4 GetBoneFromIndex(float boneIndex)
{
    // Rebuild a mat4 bone transformation from the vec4s.
    // We only transmit the top three values of each column, packed into 3 vec4s
    int boneId = int(boneIndex * 3.0);

    return mat4(
        vec4(uBoneTransforms[boneId].xyz, 0.0),
        vec4(uBoneTransforms[boneId].w, uBoneTransforms[boneId + 1].xy, 0.0),
        vec4(uBoneTransforms[boneId + 1].zw, uBoneTransforms[boneId + 2].x, 0.0),
        vec4(uBoneTransforms[boneId + 2].yzw, 1.0));
}

#endif  // USE_BONE_HALF_FLOATS
#endif  // ANIMATION_VERTEX
