#include "UndefineInOut.inc"
#include "TextureCubeUtils.inc"
#include "lighting/ShadowUtils.inc"


#if defined(SHADER_TYPE_PIXEL)

#if defined(NXT_MOBILE)
    #if defined(SHADOWS_QUALITY_LOW)
        #define POINT_SHADOW_FADE_STANDARD 1
        #define POINT_SHADOW_MAP_FILTER_BOX2X2 1
    #elif defined(SHADOWS_QUALITY_MED)
        #define POINT_SHADOW_FADE_STANDARD 1
        #define POINT_SHADOW_MAP_FILTER_BOX2X2 1
    #elif defined(SHADOWS_QUALITY_HIGH)
        #define POINT_SHADOW_FADE_SMOOTH 1
        #define POINT_SHADOW_MAP_FILTER_BOX4X4_APPROX 1
    #elif defined(SHADOWS_QUALITY_ULTRA)
        #define POINT_SHADOW_FADE_SMOOTH 1
        #define POINT_SHADOW_MAP_FILTER_BOX4X4 1
    #else
        #error Shadow quality not defined
    #endif
#else
    #if defined(SHADOWS_QUALITY_LOW)
        #define POINT_SHADOW_FADE_STANDARD 1
        #define POINT_SHADOW_MAP_FILTER_BOX2X2 1
    #elif defined(SHADOWS_QUALITY_MED)
        #define POINT_SHADOW_FADE_STANDARD 1
        #define POINT_SHADOW_MAP_FILTER_BOX4X4_APPROX 1
    #elif defined(SHADOWS_QUALITY_HIGH)
        #define POINT_SHADOW_FADE_SMOOTH 1
        #define POINT_SHADOW_MAP_FILTER_BOX4X4 1
    #elif defined(SHADOWS_QUALITY_ULTRA)
        #define POINT_SHADOW_FADE_SMOOTH 1
        #define POINT_SHADOW_MAP_FILTER_DISK8 1
    #else
        #error Shadow quality not defined
    #endif
#endif

/** @return Point light shadow attenuation (i.e. amount in light) */
float PointLightShadowAtten(
    highp vec3 toLightVec,
    highp float pixelViewDepth,
    highp float fragmentDstToLightSqrd,
    highp float lightInvRadiusSqrd,
    PointLightSMConsts pointLightSMConsts,
    SHADOW_SAMPLER shadowDepthTexture)
{
    float amountInLight = 1.0;

    toLightVec.z *= -1.0; // Reverse the z axis as UVs are flipped in GL

    // Calculate the shadow map UVs
    vec4 uvExtents = vec4(0.0, 0.0, 0.0, 0.0);
    vec3 faceTexCoords = Vec3To2DCubeMapTexCoords(uvExtents, -toLightVec, pointLightSMConsts.AtlasFaceUVScale, pointLightSMConsts.AtlasFaceUVExtents);
    vec4 texCoords = vec4(faceTexCoords.x, faceTexCoords.y, (fragmentDstToLightSqrd * lightInvRadiusSqrd) + pointLightSMConsts.FilterBiasParams.z, 0.0);

    #if defined(POINT_SHADOW_MAP_FILTER_BOX2X2)
        amountInLight = ShadowDepthMapFilter2x2Box(shadowDepthTexture, texCoords, pointLightSMConsts.AtlasFaceUVTexelSize, uvExtents);
    #elif defined(POINT_SHADOW_MAP_FILTER_BOX4X4_APPROX)
        amountInLight = ShadowDepthMapFilter4x4BoxApprox(shadowDepthTexture, texCoords, pointLightSMConsts.AtlasFaceUVTexelSize, uvExtents);
    #elif defined(POINT_SHADOW_MAP_FILTER_BOX4X4)
        amountInLight = ShadowDepthMapFilter4x4Box(shadowDepthTexture, texCoords, pointLightSMConsts.AtlasFaceUVTexelSize, uvExtents);
    #elif defined(POINT_SHADOW_MAP_FILTER_DISK8)
        amountInLight = ShadowDepthMapDiskFilter8Tap(shadowDepthTexture, texCoords, pointLightSMConsts.AtlasFaceUVTexelSize, uvExtents);
    #elif defined(POINT_SHADOW_MAP_FILTER_DISK12)
        amountInLight = ShadowDepthMapDiskFilter12Tap(shadowDepthTexture, texCoords, pointLightSMConsts.AtlasFaceUVTexelSize, uvExtents);
    #else
        #error No opaque point light shadow filter defined
    #endif

    // Fade must come after final attenuation
    #if defined(POINT_SHADOW_FADE_STANDARD)
        amountInLight = ShadowFade(amountInLight, pixelViewDepth, pointLightSMConsts.EnabledFadeViewParams.y, pointLightSMConsts.EnabledFadeViewParams.w);
    #elif defined(POINT_SHADOW_FADE_SMOOTH)
        amountInLight = ShadowFadeSmooth(amountInLight, pixelViewDepth, pointLightSMConsts.EnabledFadeViewParams.y, pointLightSMConsts.EnabledFadeViewParams.z);
    #else
        #error No shadow fade function specified
    #endif

    // Attenuate according to shadow strength
    amountInLight = min(1.0, amountInLight + (1.0 - pointLightSMConsts.FilterBiasParams.w));

    return amountInLight;
}

#endif // SHADER_TYPE_PIXEL

#include "DefineInOut.inc"
