#ifndef STIPPLE_TRANSPARENCY_UTILS_INC
#define STIPPLE_TRANSPARENCY_UTILS_INC

#if defined(STIPPLE_TRANSPARENCY_CLIP_NEAR) || defined(STIPPLE_TRANSPARENCY_CLIP_FAR) || defined(STIPPLE_TRANSPARENCY_ALPHA)

#include "Shared/StippleCommon.inc"
#include "Shared/NoiseUtils.inc"

#define STIPPLE_TRANSPARENCY_ENABLED

#if defined(STIPPLE_TRANSPARENCY_CLIP_NEAR) || defined(STIPPLE_TRANSPARENCY_CLIP_FAR)
uniform vec4 uStippleTransparencyClipParams; // x = near fade start threshold,  y = 1 / near fade range, z = far fade start threshold, w = 1 / far fade range
#endif

#include "UndefineInOut.inc"
float GetStippleTransparencyAlpha(float viewSpaceDepth, inout float alpha)
{
    float stippleAlpha = 1.0;

#if defined(STIPPLE_TRANSPARENCY_CLIP_NEAR)
    float nearFadeAlpha = (viewSpaceDepth - (uZBufferParams.w + uStippleTransparencyClipParams.x)) * uStippleTransparencyClipParams.y;
    stippleAlpha *= clamp(nearFadeAlpha, 0.0, 1.0);
#endif

#if defined(STIPPLE_TRANSPARENCY_CLIP_FAR)
    // The extra abs is necessary to deal with different views handling depth ranges differently
    float farFadeAlpha = 1.0 - (viewSpaceDepth - (abs(uZBufferParams.z) - uStippleTransparencyClipParams.z)) * uStippleTransparencyClipParams.w;
    stippleAlpha *= clamp(farFadeAlpha, 0.0, 1.0);
#endif

#if defined(STIPPLE_TRANSPARENCY_ALPHA)
    stippleAlpha *= clamp(alpha + 0.005, 0.0, 1.0); // Add a tiny amount of alpha to avoid precision/rounding issues
    alpha = 1.0;
#endif

    return stippleAlpha;
}

/**
 *  Checks if a pixel is visible using stipple transparency, based on the viewspace depth at that pixel and its base alpha.
 */
bool IsStipplePixelVisible(highp vec3 viewSpacePosition, highp vec2 screenPixelPosition, inout float alpha)
{
    float viewSpaceDepth = GetStippleViewSpaceDepthFromPos(viewSpacePosition);
    return GetStippleTransparencyAlpha(viewSpaceDepth, alpha) > GetInterleavedGradientNoise(screenPixelPosition);
}
#include "DefineInOut.inc"

#endif

#endif // STIPPLE_TRANSPARENCY_UTILS_INC
