#if defined(VIEWPORTMAP)

vec3 SampleViewportMapColour(
    highp vec2 screenPos,
    highp vec4 uvScaleAndAnim)
{
    // Convert to normalised device coordinates
    screenPos = screenPos * uViewportLookupScale.xy;

    // The screen position fits the viewport aspect ratio, so we need to adjust to make it match the sampler aspect ratio instead
    vec2 texSize;
#if defined(OGLES2_BACKEND)
    texSize = uViewportMapTextureSize.xy;
#else
    texSize = vec2(textureSize(uViewportMap, 0));
#endif
    // NOTE: We adjust the X to match the aspect ratio, leaving the Y as 0.0-1.0
    screenPos.x *= (uViewportLookupScale.z * uViewportLookupScale.y) / (texSize.x / texSize.y);

    // Apply the UV scale and animation
    screenPos *= uvScaleAndAnim.xy;
    // Copy to local highp to workaround precision bug
    highp float textureAnimationTime = uTextureAnimationTime;
    screenPos += uvScaleAndAnim.zw * textureAnimationTime;

    // NOTE: Non-skybox textures have mips, so we could blur them if we wanted to
    return texture2DLodCompat(uViewportMap, screenPos, 0.0).rgb;
}

#endif
