#ifndef APPLY_VOLUMETRICS_INC
#define APPLY_VOLUMETRICS_INC

#include "Shared/VolumetricScattering/VolumetricFunctions.inc"

#if defined(VOLUMETRIC_SCATTERING) && defined(SUNLIGHT_DIRECT_LIGHTING)

uniform vec3 uVolumetricLitFogColour;
uniform vec3 uVolumetricUnlitFogColour;
uniform mat4 uVolumetricDitherMat;

#include "UndefineInOut.inc"
void GetInAndOutScattering(vec4 scatteringResults, out vec3 extinction, out vec3 inScattering)
{
    vec3 litColour = uSunColour * uVolumetricLitFogColour;
    vec3 unlitColour = uAmbientColour * uVolumetricUnlitFogColour;
    extinction = vec3(exp(-scatteringResults.y));
    inScattering = (scatteringResults.x * litColour) + (scatteringResults.z * unlitColour);
}
#include "DefineInOut.inc"

/**
    Applies scatteringResults to the incoming originalColour
 */
vec4 ApplyVolumetricScattering(vec4 originalColour, vec4 scatteringResults)
{
    vec3 extinction = vec3(1.0);
    vec3 inScattering = vec3(0.0);
    GetInAndOutScattering(scatteringResults, extinction, inScattering);
    return vec4((originalColour.rgb * extinction) + inScattering, originalColour.a);
}

float CalculateScatteringOffset(vec2 screenPos)
{
    vec2 interleavedPos = vec2(floor(mod(screenPos.xy, 4.0)));
    return uVolumetricDitherMat[int(interleavedPos.x)][int(interleavedPos.y)];
}

/**
    Helper that combines both scattering results and volumetrics into one function from a screen pos

    @todo This currently doesn't compile due to strange errors resulting from the GetScatteredInRay2() function call
 *
vec4 ApplyVolumetricScattering2(vec2 screenPos, vec4 originalColour, int rayMarchSteps, vec3 viewDir, float cameraDistance)
{
    float offset = CalculateScatteringOffset(screenPos);
    vec4 scattering = GetScatteredInRay2(rayMarchSteps, viewDir, cameraDistance, offset);
    return ApplyVolumetricScattering(originalColour, scattering);
}
*/

#endif

#endif // APPLY_VOLUMETRICS_INC
