void main()
{
    vec3 norm = normalize(vViewSpaceNormal);
    vec4 fragColour;
    #if defined(SKYBOX_TRANSITION)
        // Only sample first sky box texture if we're transitioning
        vec3 firstColour = textureCubeSRGB(uSkyBoxMap1, norm).rgb;
        vec3 secondColour = textureCubeSRGB(uSkyBoxMap2, norm).rgb;
        if (!uSkyBoxUseFirst)
        {
           firstColour = secondColour.xyz;
        }
        if (!uSkyBoxUseSecond)
        {
           secondColour = firstColour.xyz;
        }
        fragColour = vec4(mix(firstColour, secondColour, uSkyBoxInterpolant), 1.0);
    #else
        fragColour = vec4(textureCubeSRGB(uSkyBoxMap2, norm).rgb, 1.0);
    #endif

    #if defined(LIGHT_SCATTERING) || defined(FOG_DISTANCE)
        vec3 nViewVector = norm;
        nViewVector.z = -nViewVector.z;
    #endif

    //Light scattering
    #if defined(LIGHT_SCATTERING) && defined(SKYBOX_LIGHT_SCATTERING)
        vec3 outScattering, inScattering;

        ComputeInOutScattering(nViewVector, vViewVectorAndLength.w, uInvSunDirection, outScattering, inScattering);
        vec3 scatteredFragColour = ApplyInOutScattering(fragColour.rgb, outScattering, inScattering);
        fragColour.rgb = scatteredFragColour;
    #endif

    #if defined(FOG_DISTANCE)
        fragColour.rgb = mix(fragColour.rgb, uFogColour.rgb, FogBasedOnAngle(nViewVector));
    #endif
    #if defined(LIGHT_SCATTERING) && defined(FOG_DISTANCE)
        fragColour.xyz += pow(max(dot(uInvSunDirection, nViewVector), 0.0), 128.0 * uFogParams.z) * min(uFogParams.z * nViewVector.y, 1.0);
    #endif
    fragColour.rgb += vec3(uExposure);
    // fragColour.rgb *= (vec3(1.0, 1.0, 1.0) * uExposure);  // Alternative way of doing the overbright. It doesn't wash out so much, but can leave too much blue in
    gl_FragColor = fragColour;
}
