#include "Shared/DistanceFog/Uniforms.inc"
#include "Shared/DistanceFog/Functions.inc"

void main()
{
    vec4 worldPos = uModelMatrix * vec4(aVertexPosition, 1.0);
    gl_Position = uViewProjMatrix * worldPos;
    vWorldPos_ViewSpaceDepth.xyz = worldPos.xyz;

    vVertexAlbedo = aVertexColour;
#if defined(GAMMA_CORRECT_INPUTS) && !defined(TEXTURE_ATLAS)
    vVertexAlbedo.rgb = SRGBToLinear(vVertexAlbedo.rgb);
#endif
    vVertexAlbedo.a += uFade;

#if defined(SUNLIGHT_DIRECT_LIGHTING) || defined(DEBUG_NORMALS)
    vWorldNormal = normalize(Mat4ToMat3(uModelMatrix) * aVertexNormal_FogProportion.xyz);
#endif

#if defined(IRRADIANCE_LIGHTING) && !defined(NORMAL_MAP)
    vAmbientColour = uAmbientColour * EvaluateSHLighting2ndOrder(vWorldNormal, uIrradianceSHCoefs);
#endif

#if defined(TEXTURE_ATLAS)
    vMaterialSettingsSlots1D = vec4(
        aMaterialSettingsSlotXY_MaterialSettingsSlotXY2.x + aMaterialSettingsSlotXY_MaterialSettingsSlotXY2.y * MATERIAL_SETTINGS_SLOTS_DIMENSION_COUNT_X,
        aMaterialSettingsSlotXY_MaterialSettingsSlotXY2.z + aMaterialSettingsSlotXY_MaterialSettingsSlotXY2.w * MATERIAL_SETTINGS_SLOTS_DIMENSION_COUNT_X,
        aMaterialSettingsSlotXY3.x + aMaterialSettingsSlotXY3.y * MATERIAL_SETTINGS_SLOTS_DIMENSION_COUNT_X, 0.0);
    vTextureScale = aTextureScale;
    vTextureWeight = aTextureWeight;
#endif
    // MaterialProperties is packed from TerrainPatch.cpp::AddVertex.  x,y,z each contain a packed whiteness and brightness value for one of the 3 materials.  W contains a bitfield of 3 emissive flags and 3 specular flags.
    // Since the aMaterialProperties attribute is an un-normalised UBYTE4, the range for each component is 0 - 255.  When put into vMaterialProperties, each component then becomes a float with the same values.
    // When the values are interpolated across the triangle, some error gets introduced that can break the packing done.
    // We can use a partial unit (in this case 0.25) so that the interpolator fixed point error will not bring the higher part of the packed value down enough to interact with lower values.
    // We reverse this small epsilon in the fragment shader
    vMaterialProperties = aMaterialProperties + vec4(0.25);

#if defined(LIGHT_SCATTERING) || defined(SUNLIGHT_SHADOWS) || defined(FOG_DISTANCE)
    vec3 viewVector = worldPos.xyz - uCameraPosition;
    float cameraDistance = length(viewVector);
    vec3 viewDirection = viewVector / cameraDistance;
#endif

    //Shadow mapping
#if defined(SUNLIGHT_SHADOWS)
    // Compute depth in camera space
    // Camera plane distance more accurate and must be used for cascade interval select to work without artifacts
    vec3 camLook = vec3(uViewMatrix[0][2], uViewMatrix[1][2], uViewMatrix[2][2]);
    vWorldPos_ViewSpaceDepth.w = abs(dot(viewVector.xyz, camLook));
#endif

//Light scattering
#if (defined(FOG_DISTANCE) || (defined(SUNLIGHT_DIRECT_LIGHTING) && defined(LIGHT_SCATTERING))) && !defined(OGLES2_BACKEND)
    #if defined(LIGHT_SCATTERING) && defined(SUNLIGHT_DIRECT_LIGHTING)
        ComputeInOutScattering(viewDirection, cameraDistance, uInvSunDirection, vOutScattering, vInScattering);
    #else
        vOutScattering = vec3(1.0);
        vInScattering = vec3(0.0);
    #endif

    #if defined(FOG_DISTANCE)
        float fogFactor = FogBasedOnDistance(cameraDistance);
        vInScattering = mix(vInScattering, uFogColour.rgb, fogFactor);
        vOutScattering *= 1.0 - fogFactor;
    #endif
#endif
}
