#include "ColourCorrection.inc"

uniform float uMinBlackLum;
uniform float uMaxWhiteLum;
uniform float uExposureKey;
uniform float uMinAutoExposure;
uniform float uMaxAutoExposure;
#ifdef BLUE_SHIFT_ENABLED
    uniform float uBlueShiftCoef;
#endif
uniform float uBloomScale;

in vec2 vTexCoord;

uniform sampler2D sSceneHDRTex;
uniform sampler2D sAvgLumTex;
uniform sampler2D sBloomTex;

void main()
{
    vec4 originalColourHDR = texture2DLodCompat(sSceneHDRTex, vTexCoord, 0.0);
    vec3 colour = originalColourHDR.rgb;

#ifdef BLOOM_ENABLED
    // Apply bloom effect before tone mapping so that bright areas that are further brightened up by nearby areas blooming are softly converging to the LDR maximum
    colour += texture2DLodCompat(sBloomTex, vTexCoord, 0.0).rgb * uBloomScale;
#endif

#ifdef TONE_MAPPING_ENABLED
    float avgLum = UnpackLuminanceExpFloat(texture2DLodCompat(sAvgLumTex, vec2(0.0, 0.0), 0.0));
    #ifdef BLUE_SHIFT_ENABLED
        // Apply blue shift
        colour = BlueShift(colour, avgLum, uExposureKey, uBlueShiftCoef);
    #endif
    // Apply tone mapping effect before bloom for non-HDR bloom texture
    colour = ToneMap(colour, avgLum, uMinBlackLum, uMaxWhiteLum, uExposureKey, uMinAutoExposure, uMaxAutoExposure);
#endif // TONE_MAPPING_ENABLED

    colour = clamp(colour, 0.0, 1.0);

#ifdef OUTPUT_SRGB
    colour = LinearToSRGBRunescape(colour);
#endif

#ifdef COLOUR_CORRECTION_ENABLED
    //Apply colour correction last
    colour = ColourCorrect(colour);
#endif

    gl_FragColor = vec4(colour, originalColourHDR.a);
}
