#ifndef UNPACK_UTILS_INC
#define UNPACK_UTILS_INC

#include "Common.inc"


//[ //The obfuscator doesn't like functions with multiple returns
highp float UnpackRGBAToFloat(highp vec4 valueToUnpack)
{
#if defined(USE_MOD_PACK) || defined(USE_FRACT_PACK)
    // When dot-product with the packed values, will produce a value in the range 0-1
    const highp vec4 bitShifts = vec4(1.0 / (256.0 * 256.0 * 256.0), 1.0 / (256.0 * 256.0), 1.0 / 256.0, 1.0);
    return dot(valueToUnpack, bitShifts);
#endif

#ifdef USE_ARAS_PACK
    const highp vec4 bitShifts = vec4(1.0, 1.0 / 255.0, 1.0 / 65025.0, 1.0 / 16581375.0);
    return dot(valueToUnpack, bitShifts);
#endif
}
//]

/** Unpack 32-bit integer colour to float */
/** @todo What happens when this is negative? */
vec3 ColourUnpack(highp float colour)
{
    vec3 unpackedColour;
    unpackedColour.r = floor(colour / 256.0 / 256.0);
    unpackedColour.g = floor((colour - unpackedColour.r * 256.0 * 256.0) / 256.0);
    unpackedColour.b = floor(colour - unpackedColour.r * 256.0 * 256.0 - unpackedColour.g * 256.0);
    return unpackedColour / 256.0;
}

/** Unpack normal from vec2<uint16> "Spherical environment mapping" */
vec3 NormalUnpackSphereMap(vec2 compressedNormal)
{
    vec4 fencNormal = vec4(compressedNormal.x / 32767.0 - 1.0, compressedNormal.y / 32767.0 - 1.0, 1.0, -1.0) ;
    float lNormal = dot(fencNormal.xyz, -fencNormal.xyw);
    fencNormal.xy *= sqrt(lNormal);
    fencNormal.z = lNormal;
    return fencNormal.xyz * 2.0 + vec3(0.0, 0.0, -1.0);
}

highp float UnpackRGBAToIntegerFloat(highp vec4 valueToUnpack)
{
    // valueToUnpack can be slightly irregular on some hardware (as it often comes from a sampled texture) which means
    // this function can output a non-integer value - floor() the channel values after scaling to 255 to avoid this
    return (floor(valueToUnpack.x * 255.0 + 0.5) * 256.0 * 256.0 * 256.0) + (floor(valueToUnpack.y * 255.0 + 0.5) * 256.0 * 256.0) + (floor(valueToUnpack.z * 255.0 + 0.5) * 256.0) + floor(valueToUnpack.w * 255.0 + 0.5);
}

highp float UnpackRGBAToIntegerFloat16(highp vec2 valueToUnpack)
{
    // valueToUnpack can be slightly irregular on some hardware (as it often comes from a sampled texture) which means
    // this function can output a non-integer value - floor() the channel values after scaling to 255 to avoid this
    return (floor(valueToUnpack.x * 255.0 + 0.5) * 256.0) + floor(valueToUnpack.y * 255.0 + 0.5);

    // Previous implementation for reference:
    //const highp vec4 bitShift = vec4(256.0 * 256.0 * 256.0 * 255.0, 256.0 * 256.0 * 255.0, 256.0 * 255.0, 255.0);
    //return dot(valueToUnpack, bitShift);
}

highp int UnpackRGBAToInt(vec4 valueToUnpack)
{
    return int(UnpackRGBAToIntegerFloat(valueToUnpack));
}

highp vec4 UnpackFloatToRGBA(highp float valueToUnpack)
{
    const highp vec4 bitShift = vec4(1.0, 255.0, 65025.0, 16581375.0);
    const highp vec4 bitMask = vec4(vec3(1.0 / 255.0), 0.0);
    highp vec4 result = fract(valueToUnpack * bitShift);
    result -= result.xxyz * bitMask;
    return result;
}

// Unpack colour vec2 to 16 bit float (always an integer)
// Note this can return a value +1 from the intended value due to undefined rounding when normalising
// Any code which relies on, for example, middle value = 0 should use a +/- 1 deadzone
highp float UnpackVec2ToFloat(highp vec2 valueToUnpack)
{
    // valueToUnpack can be slightly irregular on some hardware (as it often comes from a sampled texture) which means
    // this function can output a non-integer value - floor() the channel values after scaling to 255 to avoid this
    return (floor(valueToUnpack.x * 255.0 + 0.5) * 256.0) + floor(valueToUnpack.y * 255.0 + 0.5);

    // Previous implementation for reference:
    //const highp vec2 bitShift = vec2(256.0 * 255.0, 255.0);
    //return dot(valueToUnpack, bitShift);
}

#endif
